/*
 * Copyright (c) 2005-2008 Flamingo Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Flamingo Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.flamingo.common;

import java.awt.AWTEvent;
import java.awt.EventQueue;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import org.jvnet.flamingo.common.icon.ResizableIcon;
import org.jvnet.flamingo.common.ui.BasicCommandButtonUI;
import org.jvnet.flamingo.common.ui.CommandButtonUI;

/**
 * Command button.
 * 
 * @author Kirill Grouchnikov
 */
public class JCommandButton extends AbstractCommandButton {
	/**
	 * The UI class ID string.
	 */
	public static final String uiClassID = "CommandButtonUI";

	/**
	 * Associated popup panel. May be <code>null</code>.
	 */
	protected JPopupPanel popupPanel;

	/**
	 * The command button kind of this button.
	 */
	protected CommandButtonKind commandButtonKind;

	/**
	 * Indicates the auto-repeat action mode. When the button is not in the
	 * auto-repeat action mode, the registered action listeners are activated
	 * when the mouse is released (just as with the base {@link AbstractButton}
	 * ). When the button is in auto-repeat mode, the registered action
	 * listeners are activated when the mouse is pressed. In addition, is the
	 * mouse is still pressed after {@link #getAutoRepeatInitialInterval()}, the
	 * action listeners will be activated every
	 * {@link #getAutoRepeatSubsequentInterval()} until the button is disabled
	 * or the mouse is released.
	 * 
	 * @see #autoRepeatInitialInterval
	 * @see #autoRepeatSubsequentInterval
	 */
	protected boolean isAutoRepeatAction;

	/**
	 * The initial interval for invoking the registered action listeners in the
	 * auto-repeat action mode.
	 * 
	 * @see #isAutoRepeatAction
	 * @see #autoRepeatSubsequentInterval
	 */
	protected int autoRepeatInitialInterval;

	/**
	 * The subsequent interval for invoking the registered action listeners in
	 * the auto-repeat action mode.
	 * 
	 * @see #isAutoRepeatAction
	 * @see #autoRepeatInitialInterval
	 */
	protected int autoRepeatSubsequentInterval;

	/**
	 * Popup model of this button.
	 */
	protected PopupButtonModel popupModel;

	/**
	 * Default popup handler for this button.
	 */
	protected PopupHandler popupHandler;

	/**
	 * Enumerates the available command button kinds.
	 * 
	 * @author Kirill Grouchnikov
	 */
	public static enum CommandButtonKind {
		/**
		 * Command button that has only action area.
		 */
		ACTION_ONLY(true, false),

		/**
		 * Command button that has only popup area.
		 */
		POPUP_ONLY(false, true),

		/**
		 * Command button that has both action and popup areas, with the main
		 * text click activating the action.
		 */
		ACTION_AND_POPUP_MAIN_ACTION(true, true),

		/**
		 * Command button that has both action and popup areas, with the main
		 * text click activating the popup.
		 */
		ACTION_AND_POPUP_MAIN_POPUP(true, true);

		/**
		 * <code>true</code> if the command button kind has an action.
		 */
		private boolean hasAction;

		/**
		 * <code>true</code> if the command button kind has a popup.
		 */
		private boolean hasPopup;

		/**
		 * Constructs a new command button kind.
		 * 
		 * @param hasAction
		 *            Indicates whether the command button kind has an action.
		 * @param hasPopup
		 *            Indicates whether the command button kind has a popup.
		 */
		private CommandButtonKind(boolean hasAction, boolean hasPopup) {
			this.hasAction = hasAction;
			this.hasPopup = hasPopup;
		}

		/**
		 * Returns indication whether this command button kind has an action.
		 * 
		 * @return <code>true</code> if the command button kind has an action,
		 *         <code>false</code> otherwise.
		 */
		public boolean hasAction() {
			return hasAction;
		}

		/**
		 * Returns indication whether this command button kind has a popup.
		 * 
		 * @return <code>true</code> if the command button kind has a popup,
		 *         <code>false</code> otherwise.
		 */
		public boolean hasPopup() {
			return hasPopup;
		}
	}

	/**
	 * Extension of the default button model that supports the
	 * {@link PopupButtonModel} interface.
	 * 
	 * @author Kirill Grouchnikov
	 */
	private static class DefaultPopupButtonModel extends DefaultButtonModel
			implements PopupButtonModel {
		/**
		 * Timer for the auto-repeat action mode.
		 */
		protected Timer autoRepeatTimer;

		/**
		 * Creates a new default popup button model.
		 */
		public DefaultPopupButtonModel() {
			super();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.jvnet.flamingo.common.PopupButtonModel#addPopupActionListener
		 * (org.jvnet.flamingo.common.PopupActionListener)
		 */
		@Override
		public void addPopupActionListener(PopupActionListener l) {
			listenerList.add(PopupActionListener.class, l);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.jvnet.flamingo.common.PopupButtonModel#removePopupActionListener
		 * (org.jvnet.flamingo.common.PopupActionListener)
		 */
		@Override
		public void removePopupActionListener(PopupActionListener l) {
			listenerList.remove(PopupActionListener.class, l);
		}

		/**
		 * Notifies all listeners that have registered interest for notification
		 * on this event type.
		 * 
		 * @param e
		 *            the <code>ActionEvent</code> to deliver to listeners
		 * @see EventListenerList
		 */
		protected void firePopupActionPerformed(ActionEvent e) {
			// Guaranteed to return a non-null array
			Object[] listeners = listenerList.getListenerList();
			// Process the listeners last to first, notifying
			// those that are interested in this event
			for (int i = listeners.length - 2; i >= 0; i -= 2) {
				if (listeners[i] == PopupActionListener.class) {
					((PopupActionListener) listeners[i + 1]).actionPerformed(e);
				}
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.DefaultButtonModel#setPressed(boolean)
		 */
		@Override
		public void setPressed(boolean b) {
			if ((isPressed() == b) || !isEnabled()) {
				return;
			}

			if (b) {
				stateMask |= PRESSED;
			} else {
				stateMask &= ~PRESSED;
			}

			if (!isPressed() && isArmed()) {
				int modifiers = 0;
				AWTEvent currentEvent = EventQueue.getCurrentEvent();
				if (currentEvent instanceof InputEvent) {
					modifiers = ((InputEvent) currentEvent).getModifiers();
				} else if (currentEvent instanceof ActionEvent) {
					modifiers = ((ActionEvent) currentEvent).getModifiers();
				}
				firePopupActionPerformed(new ActionEvent(this,
						ActionEvent.ACTION_PERFORMED, getActionCommand(),
						EventQueue.getMostRecentEventTime(), modifiers));
			}

			fireStateChanged();
		}
	}

	/**
	 * Extension of the default button model that supports the
	 * {@link PopupButtonModel} interface.
	 * 
	 * @author Kirill Grouchnikov
	 */
	private static class RepeatableButtonModel extends DefaultButtonModel {
		/**
		 * The button behind the model.
		 */
		private JCommandButton commandButton;

		/**
		 * Timer for the auto-repeat action mode.
		 */
		protected Timer autoRepeatTimer;

		/**
		 * Creates a new button model.
		 * 
		 * @param commandButton
		 *            The associated command button.
		 */
		public RepeatableButtonModel(JCommandButton commandButton) {
			super();
			this.commandButton = commandButton;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.DefaultButtonModel#setPressed(boolean)
		 */
		@Override
		public void setPressed(boolean b) {
			if ((isPressed() == b) || !isEnabled()) {
				return;
			}

			if (b) {
				stateMask |= PRESSED;
			} else {
				stateMask &= ~PRESSED;
			}

			boolean toFireFirstAction = isArmed();
			// if the button is in auto-repeat action mode, the action
			// starts firing on press-down and not on press-up
			if (commandButton.isAutoRepeatAction())
				toFireFirstAction = isPressed() && toFireFirstAction;
			else
				toFireFirstAction = !isPressed() && toFireFirstAction;

			// no action on popup only command buttons
			if (commandButton.getCommandButtonKind() == CommandButtonKind.POPUP_ONLY)
				toFireFirstAction = false;

			int modifiers = 0;
			AWTEvent currentEvent = EventQueue.getCurrentEvent();
			if (currentEvent instanceof InputEvent) {
				modifiers = ((InputEvent) currentEvent).getModifiers();
			} else if (currentEvent instanceof ActionEvent) {
				modifiers = ((ActionEvent) currentEvent).getModifiers();
			}
			final int finalModifiers = modifiers;
			if (toFireFirstAction) {
				fireActionPerformed(new ActionEvent(this,
						ActionEvent.ACTION_PERFORMED, getActionCommand(),
						EventQueue.getMostRecentEventTime(), modifiers));
			}

			if (commandButton.isAutoRepeatAction()) {
				if (toFireFirstAction) {
					// start timer
					autoRepeatTimer = new Timer(commandButton
							.getAutoRepeatSubsequentInterval(),
							new ActionListener() {
								@Override
								public void actionPerformed(ActionEvent e) {
									if (!isEnabled()) {
										// System.out.println("Stop timer");
										autoRepeatTimer.stop();
										return;
									}
									// System.out.println("Fire timer");
									fireActionPerformed(new ActionEvent(this,
											ActionEvent.ACTION_PERFORMED,
											getActionCommand(), EventQueue
													.getMostRecentEventTime(),
											finalModifiers));
								}
							});
					autoRepeatTimer.setInitialDelay(commandButton
							.getAutoRepeatInitialInterval());
					// System.out.println("Start timer");
					autoRepeatTimer.start();
				} else {
					// stop timer
					// System.out.println("Stop timer");
					if (autoRepeatTimer != null)
						autoRepeatTimer.stop();
				}
			}

			fireStateChanged();
		}
	}

	/**
	 * Creates a new command button.
	 * 
	 * @param title
	 *            Button title. May contain any number of words.
	 * @param icon
	 *            Button icon.
	 */
	public JCommandButton(String title, ResizableIcon icon) {
		super(title, icon);

		this.setActionModel(new RepeatableButtonModel(this));

		// important - handler creation must be done before setting
		// the popup model so that it can be registered to track the
		// changes
		this.popupHandler = new PopupHandler();
		this.setPopupModel(new DefaultPopupButtonModel());

		this.commandButtonKind = CommandButtonKind.ACTION_ONLY;
		this.state = ElementState.ORIG;
		this.isAutoRepeatAction = false;
		this.autoRepeatInitialInterval = 500;
		this.autoRepeatSubsequentInterval = 100;

		this.updateUI();
	}

	/**
	 * Returns the command button kind of this button.
	 * 
	 * @return Command button kind of this button.
	 */
	public CommandButtonKind getCommandButtonKind() {
		return this.commandButtonKind;
	}

	/**
	 * Sets the kind for this button.
	 * 
	 * @param commandButtonKind
	 *            The new button kind.
	 */
	public void setCommandButtonKind(CommandButtonKind commandButtonKind) {
		CommandButtonKind old = this.commandButtonKind;
		this.commandButtonKind = commandButtonKind;
		if (old != this.commandButtonKind) {
			firePropertyChange("commandButtonKind", old, this.commandButtonKind);
		}
	}

	/**
	 * Resets the UI property to a value from the current look and feel.
	 * 
	 * @see JComponent#updateUI
	 */
	@Override
	public void updateUI() {
		if (UIManager.get(getUIClassID()) != null) {
			setUI((CommandButtonUI) UIManager.getUI(this));
		} else {
			setUI(new BasicCommandButtonUI());
		}
		if (this.popupPanel != null) {
			SwingUtilities.updateComponentTreeUI(this.popupPanel);
			this.popupPanel.updateMaxDimension();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#getUIClassID()
	 */
	@Override
	public String getUIClassID() {
		return uiClassID;
	}

	/**
	 * Returns the associated popup panel.
	 * 
	 * @return The associated popup panel.
	 */
	public JPopupPanel getPopupPanel() {
		return popupPanel;
	}

	/**
	 * Sets new popup panel for <code>this</code> button.
	 * 
	 * @param panel
	 *            New popup panel for <code>this</code> button.
	 */
	public void setPopupPanel(JPopupPanel panel) {
		JPopupPanel old = this.popupPanel;
		this.popupPanel = panel;
		if (old != this.popupPanel) {
			firePropertyChange("popupPanel", old, this.popupPanel);
		}
	}

	/**
	 * Adds a popup menu listener to this button. All registered listeners will
	 * be called when the "popup area" of this button is activated, providing a
	 * convenient way to populate the popup menu without creating, positioning
	 * or tracking the lifecycle of the popup menu itself. Note that if
	 * {@link #getPopupPanel()} returns a non-<code>null</code> result, the
	 * popup click will show that panel and not invoke any of the registered
	 * listeners.
	 * 
	 * @param l
	 *            The popup menu listener to be added
	 */
	public void addPopupMenuListener(PopupMenuListener l) {
		listenerList.add(PopupMenuListener.class, l);
	}

	/**
	 * Removes an <code>PopupActionListener</code> from the button.
	 * 
	 * @param l
	 *            the listener to be removed
	 */
	public void removePopupActionListener(PopupMenuListener l) {
		listenerList.remove(PopupMenuListener.class, l);
	}

	/**
	 * Sets the auto-repeat action indication.
	 * 
	 * @param isAutoRepeatAction
	 *            If <code>true</code>, pressing the button will activate
	 *            auto-repeat action mode. When the button is not in the
	 *            auto-repeat action mode, the registered action listeners are
	 *            activated when the mouse is released (just as with the base
	 *            {@link AbstractButton}). When the button is in auto-repeat
	 *            mode, the registered action listeners are activated when the
	 *            mouse is pressed. In addition, is the mouse is still pressed
	 *            after {@link #getAutoRepeatInitialInterval()}, the action
	 *            listeners will be activated every
	 *            {@link #getAutoRepeatSubsequentInterval()} until the button is
	 *            disabled or the mouse is released.
	 * @see #setAutoRepeatActionIntervals(int, int)
	 * @see #isAutoRepeatAction()
	 */
	public void setAutoRepeatAction(boolean isAutoRepeatAction) {
		this.isAutoRepeatAction = isAutoRepeatAction;
	}

	/**
	 * Sets the intervals for the auto-repeat action mode.
	 * 
	 * @param initial
	 *            The initial interval for invoking the registered action
	 *            listeners in the auto-repeat action mode.
	 * @param subsequent
	 *            The subsequent interval for invoking the registered action
	 *            listeners in the auto-repeat action mode.
	 * @see #setAutoRepeatAction(boolean)
	 * @see #isAutoRepeatAction()
	 * @see #getAutoRepeatInitialInterval()
	 * @see #getAutoRepeatSubsequentInterval()
	 */
	public void setAutoRepeatActionIntervals(int initial, int subsequent) {
		this.autoRepeatInitialInterval = initial;
		this.autoRepeatSubsequentInterval = subsequent;
	}

	/**
	 * Returns indication whether the button is in auto-repeat action mode.
	 * 
	 * @return <code>true</code> if the button is in auto-repeat action mode,
	 *         <code>false</code> otherwise.
	 * @see #setAutoRepeatAction(boolean)
	 * @see #setAutoRepeatActionIntervals(int, int)
	 * @see #getAutoRepeatInitialInterval()
	 * @see #getAutoRepeatSubsequentInterval()
	 */
	public boolean isAutoRepeatAction() {
		return isAutoRepeatAction;
	}

	/**
	 * Returns the initial interval for invoking the registered action listeners
	 * in the auto-repeat action mode.
	 * 
	 * @return The initial interval for invoking the registered action listeners
	 *         in the auto-repeat action mode.
	 * @see #setAutoRepeatActionIntervals(int, int)
	 * @see #setAutoRepeatAction(boolean)
	 * @see #isAutoRepeatAction()
	 * @see #getAutoRepeatSubsequentInterval()
	 */
	public int getAutoRepeatInitialInterval() {
		return autoRepeatInitialInterval;
	}

	/**
	 * Returns the subsequent interval for invoking the registered action
	 * listeners in the auto-repeat action mode.
	 * 
	 * @return The subsequent interval for invoking the registered action
	 *         listeners in the auto-repeat action mode.
	 * @see #setAutoRepeatActionIntervals(int, int)
	 * @see #setAutoRepeatAction(boolean)
	 * @see #isAutoRepeatAction()
	 * @see #getAutoRepeatInitialInterval()
	 */
	public int getAutoRepeatSubsequentInterval() {
		return autoRepeatSubsequentInterval;
	}

	/**
	 * Returns the popup model of this button.
	 * 
	 * @return The popup model of this button.
	 */
	public PopupButtonModel getPopupModel() {
		return this.popupModel;
	}

	/**
	 * Sets the new popup model for this button.
	 * 
	 * @param newModel
	 *            The new popup model for this button.
	 */
	public void setPopupModel(PopupButtonModel newModel) {

		PopupButtonModel oldModel = getPopupModel();

		if (oldModel != null) {
			oldModel.removeChangeListener(this.popupHandler);
			oldModel.removeActionListener(this.popupHandler);
		}

		this.popupModel = newModel;

		if (newModel != null) {
			newModel.addChangeListener(this.popupHandler);
			newModel.addActionListener(this.popupHandler);
		}

		firePropertyChange("popupModel", oldModel, newModel);
		if (newModel != oldModel) {
			revalidate();
			repaint();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jvnet.flamingo.common.AbstractCommandButton#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(boolean b) {
		if (!b && popupModel.isRollover()) {
			popupModel.setRollover(false);
		}
		super.setEnabled(b);
		popupModel.setEnabled(b);
	}

	/**
	 * Default popup handler.
	 * 
	 * @author Kirill Grouchnikov
	 */
	class PopupHandler implements PopupActionListener, ChangeListener {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
			repaint();
		}

		public void actionPerformed(ActionEvent event) {
			firePopupActionPerformed(event);
		}
	}

	/**
	 * Notifies all listeners that have registered interest for notification on
	 * this event type. The event instance is lazily created using the
	 * <code>event</code> parameter.
	 * 
	 * @param event
	 *            the <code>ActionEvent</code> object
	 * @see EventListenerList
	 */
	protected void firePopupActionPerformed(ActionEvent event) {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		ActionEvent e = null;
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == PopupActionListener.class) {
				// Lazily create the event:
				if (e == null) {
					String actionCommand = event.getActionCommand();
					e = new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
							actionCommand, event.getWhen(), event
									.getModifiers());
				}
				((PopupActionListener) listeners[i + 1]).actionPerformed(e);
			}
		}
	}
}
