/*
 * Copyright (c) 2005-2008 Flamingo Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Flamingo Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.flamingo.common;

import java.awt.Component;
import java.awt.Graphics;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import org.jvnet.flamingo.common.ui.BasicCommandButtonStripUI;
import org.jvnet.flamingo.common.ui.CommandButtonStripUI;

/**
 * Button strip component. Provides visual appearance of a strip. The buttons in
 * the strip are either drawn horizontally with no horizontal space between them
 * or drawn vertically with no vertical space between them.
 * 
 * @author Kirill Grouchnikov
 */
public class JCommandButtonStrip extends JComponent {
	/**
	 * The UI class ID string.
	 */
	public static final String uiClassID = "CommandButtonStripUI";

	/**
	 * Element state for the buttons in this button strip. Default state is
	 * {@link ElementState#CUSTOM}.
	 */
	protected ElementState elementState;

	/**
	 * Button strip orientation.
	 * 
	 * @author Kirill Grouchnikov
	 */
	public enum StripOrientation {
		/**
		 * Horizontal strip orientation.
		 */
		HORIZONTAL,

		/**
		 * Vertical strip orientation.
		 */
		VERTICAL
	}

	/**
	 * Orientation of <code>this</code> strip.
	 */
	private StripOrientation orientation;

	/**
	 * Simple constructor. Creates horizontally-oriented strip.
	 */
	public JCommandButtonStrip() {
		this(StripOrientation.HORIZONTAL);
	}

	/**
	 * Simple constructor.
	 * 
	 * @param orientation
	 *            Orientation for this strip.
	 */
	public JCommandButtonStrip(StripOrientation orientation) {
		this.orientation = orientation;
		this.elementState = ElementState.CUSTOM;
		updateUI();
	}

	/**
	 * Sets the element state for the buttons in this button strip. This method
	 * must be called <em>before</em> adding the first command button. The
	 * default state is {@link ElementState#CUSTOM}.
	 * 
	 * @param elementState
	 *            New element state for the buttons in this button strip.
	 */
	public void setElementState(ElementState elementState) {
		if (this.getComponentCount() > 0) {
			throw new IllegalStateException(
					"Can't call this method after buttons have been already added");
		}
		this.elementState = elementState;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Container#add(java.awt.Component, java.lang.Object, int)
	 */
	@Override
	public void add(Component comp, Object constraints, int index) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Container#add(java.awt.Component, java.lang.Object)
	 */
	@Override
	public void add(Component comp, Object constraints) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Container#add(java.awt.Component, int)
	 */
	@Override
	public Component add(Component comp, int index) {
		if (!(comp instanceof AbstractCommandButton))
			throw new UnsupportedOperationException();
		this.configureCommandButton((AbstractCommandButton) comp);
		return super.add(comp, index);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Container#add(java.awt.Component)
	 */
	@Override
	public Component add(Component comp) {
		if (!(comp instanceof AbstractCommandButton))
			throw new UnsupportedOperationException();
		this.configureCommandButton((AbstractCommandButton) comp);
		return super.add(comp);
	}

	/**
	 * Configures the specified command button.
	 * 
	 * @param button
	 *            Command button to configure.
	 */
	private void configureCommandButton(AbstractCommandButton button) {
		button.setState(this.elementState, true);
		if (this.elementState == ElementState.COMPACT)
			button.setBorder(new EmptyBorder(3, 6, 3, 6));
		button.setFlat(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Container#add(java.lang.String, java.awt.Component)
	 */
	@Override
	public Component add(String name, Component comp) {
		throw new UnsupportedOperationException();
	}

	/**
	 * Sets the new UI delegate.
	 * 
	 * @param ui
	 *            New UI delegate.
	 */
	public void setUI(CommandButtonStripUI ui) {
		super.setUI(ui);
	}

	/**
	 * Resets the UI property to a value from the current look and feel.
	 * 
	 * @see JComponent#updateUI
	 */
	@Override
	public void updateUI() {
		if (UIManager.get(getUIClassID()) != null) {
			setUI((CommandButtonStripUI) UIManager.getUI(this));
		} else {
			setUI(new BasicCommandButtonStripUI());
		}
	}

	/**
	 * Returns the UI object which implements the L&F for this component.
	 * 
	 * @return a <code>ButtonStripUI</code> object
	 * @see #setUI
	 */
	public CommandButtonStripUI getUI() {
		return (CommandButtonStripUI) ui;
	}

	/**
	 * Returns the name of the UI class that implements the L&F for this
	 * component.
	 * 
	 * @return the string "ButtonStripUI"
	 * @see JComponent#getUIClassID
	 * @see UIDefaults#getUI
	 */
	@Override
	public String getUIClassID() {
		return uiClassID;
	}

	/**
	 * Returns the number of buttons in <code>this</code> strip.
	 * 
	 * @return Number of buttons in <code>this</code> strip.
	 */
	public int getButtonCount() {
		return this.getComponentCount();
	}

	/**
	 * Returns the specified button component of <code>this</code> strip.
	 * 
	 * @param index
	 *            Button index.
	 * @return The matching button.
	 */
	public AbstractCommandButton getButton(int index) {
		return (AbstractCommandButton) this.getComponent(index);
	}

	/**
	 * Checks whether the specified button is the first button in
	 * <code>this</code> strip.
	 * 
	 * @param button
	 *            Button to check.
	 * @return <code>true</code> if the specified button is the first button in
	 *         <code>this</code> strip, <code>false</code> otherwise.
	 */
	public boolean isFirst(AbstractCommandButton button) {
		return (button == this.getButton(0));
	}

	/**
	 * Checks whether the specified button is the last button in
	 * <code>this</code> strip.
	 * 
	 * @param button
	 *            Button to check.
	 * @return <code>true</code> if the specified button is the last button in
	 *         <code>this</code> strip, <code>false</code> otherwise.
	 */
	public boolean isLast(AbstractCommandButton button) {
		return (button == this.getButton(this.getButtonCount() - 1));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#paintBorder(java.awt.Graphics)
	 */
	@Override
	protected void paintBorder(Graphics g) {
		// the border will be painted in the UI delegate.
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#paintChildren(java.awt.Graphics)
	 */
	@Override
	protected void paintChildren(Graphics g) {
		// the child components (buttons) will be painted in the UI
		// delegate.
	}

	/**
	 * Returns the orientation of <code>this</code> strip.
	 * 
	 * @return Orientation of <code>this</code> strip.
	 */
	public StripOrientation getOrientation() {
		return orientation;
	}
}
