/*
 * Copyright (c) 2005-2008 Flamingo Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Flamingo Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.flamingo.common;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.BorderUIResource;
import javax.swing.plaf.UIResource;

import org.jvnet.flamingo.common.ui.BasicCommandButtonPanelUI;
import org.jvnet.flamingo.utils.FlamingoUtilities;

/**
 * Popup panel that hosts groups of icons.
 * 
 * @author Kirill Grouchnikov
 */
public class JIconPopupPanel extends JPopupPanel {
	/**
	 * Maximum dimension of <code>this</code> popup gallery.
	 */
	protected Dimension maxDimension;

	/**
	 * The internal panel that hosts the icon command buttons. Is hosted in the
	 * {@link #scroll}.
	 */
	protected JCommandButtonPanel iconPanel;

	/**
	 * The maximum number of visible button rows.
	 */
	protected int maxVisibleButtonRows;

	/**
	 * Scroll panel that hosts {@link #iconPanel}.
	 */
	protected JScrollPane scroll;

	/**
	 * Creates new a icon popup panel.
	 * 
	 * @param iconPanel
	 *            The internal panel that hosts icon command buttons.
	 * @param maxButtonColumns
	 *            The maximum number of button columns.
	 * @param maxVisibleButtonRows
	 *            The maximum number of visible button rows.
	 */
	public JIconPopupPanel(JCommandButtonPanel iconPanel, int maxButtonColumns,
			int maxVisibleButtonRows) {
		this.iconPanel = iconPanel;
		this.iconPanel.setMaxButtonColumns(maxButtonColumns);
		this.maxVisibleButtonRows = maxVisibleButtonRows;

		int maxButtonWidth = 0;
		int maxButtonHeight = 0;
		int groupCount = iconPanel.getGroupCount();
		for (int i = 0; i < groupCount; i++) {
			for (AbstractCommandButton button : iconPanel.getGroupButtons(i)) {
				maxButtonWidth = Math.max(maxButtonWidth, button
						.getPreferredSize().width);
				maxButtonHeight = Math.max(maxButtonHeight, button
						.getPreferredSize().height);
			}
		}

		updateMaxDimension();

		this.scroll = new JScrollPane(this.iconPanel,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		this.scroll.setBorder(new EmptyBorder(0, 0, 0, 0));
		this.scroll.setOpaque(false);
		this.scroll.getViewport().setOpaque(false);
		this.setLayout(new IconPopupLayout());

		// JViewport viewport = this.scroll.getViewport();
		// viewport.setPreferredSize(new Dimension(maxDimension.width
		// + this.scroll.getVerticalScrollBar().getPreferredSize().width,
		// maxDimension.height));
		// this.setPreferredSize(this.maxDimension);
		this.add(this.scroll);

		Border b = this.getBorder();
		if (b == null || b instanceof UIResource) {
			Border toSet = UIManager.getBorder("PopupGallery.border");
			if (toSet == null)
				toSet = new BorderUIResource.LineBorderUIResource(
						FlamingoUtilities.getColor(Color.gray,
								"TextField.inactiveForeground",
								"Button.disabledText",
								"ComboBox.disabledForeground"));
			this.setBorder(toSet);
		}
	}

	/**
	 * Updates the max dimension of this panel. This method is for internal use
	 * only.
	 */
	@Override
	public void updateMaxDimension() {
		if (this.iconPanel == null)
			return;
		this.iconPanel.setPreferredSize(null);
		Dimension prefIconPanelDim = this.iconPanel.getPreferredSize();
		// fix for issue 13 - respect the gaps and insets
		BasicCommandButtonPanelUI panelUI = (BasicCommandButtonPanelUI) iconPanel
				.getUI();
		int titlePanelCount = iconPanel.isToShowGroupLabels() ? 1 : 0;
		this.maxDimension = new Dimension(prefIconPanelDim.width, panelUI
				.getPreferredHeight(this.maxVisibleButtonRows, titlePanelCount));
		this.setPreferredSize(null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jvnet.flamingo.common.AbstractPopupPanel#isEmpty()
	 */
	@Override
	public boolean isEmpty() {
		return (this.iconPanel.getGroupCount() == 0);
	}

	/**
	 * Layout manager for <code>this</code> popup gallery.
	 * 
	 * @author Kirill Grouchnikov
	 */
	protected class IconPopupLayout implements LayoutManager {
		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#addLayoutComponent(java.lang.String,
		 * java.awt.Component)
		 */
		public void addLayoutComponent(String name, Component comp) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#removeLayoutComponent(java.awt.Component)
		 */
		public void removeLayoutComponent(Component comp) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#layoutContainer(java.awt.Container)
		 */
		public void layoutContainer(Container parent) {
			Insets insets = parent.getInsets();
			int left = insets.left;
			int right = insets.right;
			int top = insets.top;
			int bottom = insets.bottom;
			scroll.setBounds(left, top, parent.getWidth() - left - right,
					parent.getHeight() - top - bottom);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#minimumLayoutSize(java.awt.Container)
		 */
		public Dimension minimumLayoutSize(Container parent) {
			return this.preferredLayoutSize(parent);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#preferredLayoutSize(java.awt.Container)
		 */
		public Dimension preferredLayoutSize(Container parent) {
			Insets insets = parent.getInsets();
			int left = insets.left;
			int right = insets.right;
			int top = insets.top;
			int bottom = insets.bottom;
			Dimension controlPanelDim = iconPanel.getPreferredSize();
			if (controlPanelDim == null)
				controlPanelDim = new Dimension(0, 0);
			int w = Math.min(controlPanelDim.width, maxDimension.width) + left
					+ right;
			int h = Math.min(controlPanelDim.height, maxDimension.height) + top
					+ bottom;
			if (h == (maxDimension.height + top + bottom)) {
				int scrollBarWidth = UIManager.getInt("ScrollBar.width");
				if (scrollBarWidth == 0) {
					// Nimbus
					scrollBarWidth = new JScrollBar(JScrollBar.VERTICAL)
							.getPreferredSize().width;
				}
				w += scrollBarWidth;
				// h += 5;
			}
			return new Dimension(w, h);
		}
	}
}
