/*
 * Copyright (c) 2005-2008 Flamingo Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Flamingo Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.flamingo.common.ui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.StringTokenizer;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.BasicGraphicsUtils;

import org.jvnet.flamingo.common.*;
import org.jvnet.flamingo.common.JCommandButton.CommandButtonKind;
import org.jvnet.flamingo.common.icon.ResizableIcon;
import org.jvnet.flamingo.utils.FlamingoUtilities;
import org.jvnet.flamingo.utils.RenderingUtils;

/**
 * Basic UI for command button {@link JCommandButton}.
 * 
 * @author Kirill Grouchnikov
 */
public class BasicCommandButtonUI extends CommandButtonUI {
	/**
	 * The associated command button.
	 */
	protected AbstractCommandButton commandButton;

	/**
	 * Indication whether the mouse pointer is over the associated command
	 * button.
	 */
	protected boolean isUnderMouse;

	/**
	 * Property change listener.
	 */
	protected PropertyChangeListener propertyChangeListener;

	/**
	 * Tracks user interaction with the command button (including keyboard and
	 * mouse).
	 */
	protected BasicPopupButtonListener basicPopupButtonListener;

	/**
	 * Action area of the command button.
	 */
	protected Rectangle actionClickArea;

	/**
	 * Popup area of the command button.
	 */
	protected Rectangle popupClickArea;

	/**
	 * Client property to mark the command button to have square corners. This
	 * client property is for internal use only.
	 */
	public static final String EMULATE_SQUARE_BUTTON = "flamingo.internal.commandButton.ui.emulateSquare";

	/**
	 * Client property to mark the command button to not dispose the popups on
	 * activation.
	 * 
	 * @see #disposePopupsActionListener
	 */
	public static final String DONT_DISPOSE_POPUPS = "flamingo.internal.commandButton.ui.dontDisposePopups";

	/**
	 * This listener disposes all popup panels when button's action is
	 * activated. An example of scenario would be a command button in the popup
	 * panel of an in-ribbon gallery. When this command button is activated, the
	 * associated popup panel is dismissed.
	 * 
	 * @see #DONT_DISPOSE_POPUPS
	 */
	protected ActionListener disposePopupsActionListener;

	/**
	 * Action listener on the popup area.
	 */
	protected PopupActionListener popupActionListener;

	/**
	 * The first part of (possibly) two-lined split of {@link #commandButton}'s
	 * title.
	 */
	protected String titlePart1;

	/**
	 * The second part of (possibly) two-lined split of {@link #commandButton}'s
	 * title.
	 */
	protected String titlePart2;

	/**
	 * The "expand" action icon.
	 */
	protected ResizableIcon popupActionIcon;

	/**
	 * Indication whether the {@link #savedDimension} should be used during the
	 * layout.
	 */
	protected boolean toTakeSavedDimension;

	/**
	 * Saved dimension for speeding up the layout.
	 */
	protected Dimension savedDimension;

	/**
	 * Used to provide a LAF-consistent appearance under core LAFs.
	 */
	protected CellRendererPane buttonRendererPane;

	/**
	 * Used to provide a LAF-consistent appearance under core LAFs.
	 */
	protected AbstractButton rendererButton;

	/**
	 * Used to paint the separator between the action and popup areas.
	 */
	protected JSeparator rendererSeparator;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#createUI(javax.swing.JComponent)
	 */
	public static ComponentUI createUI(JComponent c) {
		return new BasicCommandButtonUI();
	}

	/**
	 * Creates a new UI delegate.
	 */
	public BasicCommandButtonUI() {
		this.toTakeSavedDimension = false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#installUI(javax.swing.JComponent)
	 */
	@Override
	public void installUI(JComponent c) {
		this.commandButton = (AbstractCommandButton) c;
		installDefaults();
		installComponents();
		installListeners();
		installKeyboardActions();
		this.commandButton.setLayout(createLayoutManager());
		if (this.commandButton.getState() != null)
			this.updateState(this.commandButton.getState(), false);

		this.updateCustomDimension();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#uninstallUI(javax.swing.JComponent)
	 */
	@Override
	public void uninstallUI(JComponent c) {
		c.setLayout(null);

		uninstallKeyboardActions();
		uninstallListeners();
		uninstallComponents();
		uninstallDefaults();
		this.commandButton = null;
	}

	/**
	 * Installs defaults on the associated command button.
	 */
	protected void installDefaults() {
		// this.commandButton.setRolloverEnabled(true);
		this.buttonRendererPane = new CellRendererPane();
		this.commandButton.add(buttonRendererPane);
		this.rendererButton = createRendererButton();
		this.rendererSeparator = new JSeparator();
		Font currFont = this.commandButton.getFont();
		if ((currFont == null) || (currFont instanceof UIResource)) {
			this.commandButton.setFont(this.rendererButton.getFont());
		}

		Border currBorder = this.commandButton.getBorder();
		if ((currBorder == null) || (currBorder instanceof UIResource)) {
			// create a dummy button
			JToggleButton button = new JToggleButton(new Icon() {
				@Override
				public int getIconHeight() {
					return 16;
				}

				@Override
				public int getIconWidth() {
					return 16;
				}

				@Override
				public void paintIcon(Component c, Graphics g, int x, int y) {
				}
			});
			button.setText(null);
			// Compute how many pixels the specific LAF allocates
			// around the icon. Some LAFs (such as Nimbus, for example)
			// require more pixels around the button contents. This
			// code tries to re-create the LAF-specific button margins.
			int topBottom = (button.getMinimumSize().height - 16) / 2 - 2;
			int leftRight = (button.getMinimumSize().width - 16) / 3;

			this.commandButton
					.setBorder(new BorderUIResource.EmptyBorderUIResource(
							topBottom, leftRight, topBottom, leftRight));
		}
	}

	/**
	 * Creates the renderer button.
	 * 
	 * @return The renderer button.
	 */
	protected AbstractButton createRendererButton() {
		return new JButton("");
	}

	/**
	 * Installs subcomponents on the associated command button.
	 */
	protected void installComponents() {
		this.updateTitleStrings();
		this.updatePopupActionIcon();

		ResizableIcon buttonIcon = this.commandButton.getIcon();
		if (buttonIcon instanceof AsynchronousLoading) {
			((AsynchronousLoading) buttonIcon)
					.addAsynchronousLoadListener(new AsynchronousLoadListener() {
						public void completed(boolean success) {
							if (success && (commandButton != null))
								commandButton.repaint();
						}
					});
		}

		if (this.commandButton instanceof JCommandButton) {
			if (((JCommandButton) this.commandButton).getPopupPanel() != null) {
				this.popupActionIcon = this.createPopupActionIcon();
			}
		}
	}

	/**
	 * Updates the title strings for {@link ElementState#BIG} and other relevant
	 * states.
	 */
	protected void updateTitleStrings() {
		// Break the title in two parts (the second part may be empty),
		// finding the "inflection" point. The inflection point is a space
		// character that breaks the title in two parts, such that the maximal
		// length of the first part and the second part + action label icon
		// is minimal between all possible space characters
		BufferedImage tempImage = new BufferedImage(30, 30,
				BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = (Graphics2D) tempImage.getGraphics();
		g.setFont(FlamingoUtilities.getFont(this.commandButton, "Ribbon.font",
				"Button.font", "Panel.font"));
		FontMetrics fm = g.getFontMetrics();

		String title = this.commandButton.getText();
		if (title != null) {
			StringTokenizer tokenizer = new StringTokenizer(title, " _-", true);
			if (tokenizer.countTokens() <= 1) {
				// single word
				this.titlePart1 = title;
				this.titlePart2 = "";
			} else {
				int currMaxLength = (int) fm.getStringBounds(
						this.commandButton.getText(), g).getWidth();
				int actionIconWidth = (this.popupActionIcon == null) ? 0
						: 4 + this.popupActionIcon.getIconWidth();

				String currLeading = "";
				while (tokenizer.hasMoreTokens()) {
					currLeading += tokenizer.nextToken();
					String part1 = currLeading;
					String part2 = title.substring(currLeading.length());

					int len1 = (int) fm.getStringBounds(part1, g).getWidth();
					int len2 = (int) fm.getStringBounds(part2, g).getWidth()
							+ actionIconWidth;
					int len = Math.max(len1, len2);

					if (currMaxLength > len) {
						currMaxLength = len;
						this.titlePart1 = part1;
						this.titlePart2 = part2;
					}
				}
			}
		} else {
			this.titlePart1 = "";
			this.titlePart2 = "";
		}
	}

	/**
	 * Installs listeners on the associated command button.
	 */
	protected void installListeners() {
		this.basicPopupButtonListener = createButtonListener(this.commandButton);
		if (this.basicPopupButtonListener != null) {
			this.commandButton.addMouseListener(this.basicPopupButtonListener);
			this.commandButton
					.addMouseMotionListener(this.basicPopupButtonListener);
			this.commandButton.addFocusListener(this.basicPopupButtonListener);
			this.commandButton.addChangeListener(this.basicPopupButtonListener);
		}

		this.propertyChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (AbstractButton.ICON_CHANGED_PROPERTY.equals(evt
						.getPropertyName())) {
					Icon newIcon = (Icon) evt.getNewValue();
					if (newIcon instanceof AsynchronousLoading) {
						((AsynchronousLoading) newIcon)
								.addAsynchronousLoadListener(new AsynchronousLoadListener() {
									public void completed(boolean success) {
										if (success) {
											commandButton.repaint();
										}
									}
								});
					}
				}
				if ("commandButtonKind".equals(evt.getPropertyName())) {
					updatePopupActionIcon();
				}
				if ("popupPanel".equals(evt.getPropertyName())) {
					updatePopupActionIcon();
				}
				if ("customDimension".equals(evt.getPropertyName())) {
					updateCustomDimension();
				}
				if ("text".equals(evt.getPropertyName())
						|| "font".equals(evt.getPropertyName())) {
					updateTitleStrings();
				}

				if ("popupModel".equals(evt.getPropertyName())) {
					// rewire the popup action listener on the new popup model
					PopupButtonModel oldModel = (PopupButtonModel) evt
							.getOldValue();
					PopupButtonModel newModel = (PopupButtonModel) evt
							.getNewValue();

					if (oldModel != null) {
						oldModel.removePopupActionListener(popupActionListener);
						popupActionListener = null;
					}

					if (newModel != null) {
						popupActionListener = createPopupActionListener();
						newModel.addPopupActionListener(popupActionListener);
					}
				}
			}
		};
		this.commandButton
				.addPropertyChangeListener(this.propertyChangeListener);

		this.disposePopupsActionListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (!Boolean.TRUE.equals(commandButton
						.getClientProperty(DONT_DISPOSE_POPUPS))) {
					PopupPanelManager.defaultManager().hidePopups(null);
				}
			}
		};
		this.commandButton.addActionListener(this.disposePopupsActionListener);

		if (this.commandButton instanceof JCommandButton) {
			this.popupActionListener = this.createPopupActionListener();
			((JCommandButton) this.commandButton).getPopupModel()
					.addPopupActionListener(this.popupActionListener);
		}

	}

	/**
	 * Creates the icon for the popup area.
	 * 
	 * @return The icon for the popup area.
	 */
	protected ResizableIcon createPopupActionIcon() {
		return FlamingoUtilities.getCommandButtonPopupActionIcon();
	}

	/**
	 * Creates the button listener for the specified command button.
	 * 
	 * @param b
	 *            Command button.
	 * @return The button listener for the specified command button.
	 */
	protected BasicPopupButtonListener createButtonListener(
			AbstractCommandButton b) {
		return new BasicPopupButtonListener();
	}

	/**
	 * Installs the keyboard actions on the associated command button.
	 */
	protected void installKeyboardActions() {
		if (this.basicPopupButtonListener != null) {
			basicPopupButtonListener.installKeyboardActions(this.commandButton);
		}
	}

	/**
	 * Uninstalls defaults from the associated command button.
	 */
	protected void uninstallDefaults() {
		this.commandButton.remove(this.buttonRendererPane);
		this.buttonRendererPane = null;
	}

	/**
	 * Uninstalls subcomponents from the associated command button.
	 */
	protected void uninstallComponents() {
	}

	/**
	 * Uninstalls listeners from the associated command button.
	 */
	protected void uninstallListeners() {
		if (this.basicPopupButtonListener != null) {
			this.commandButton
					.removeMouseListener(this.basicPopupButtonListener);
			this.commandButton
					.removeMouseListener(this.basicPopupButtonListener);
			this.commandButton
					.removeMouseMotionListener(this.basicPopupButtonListener);
			this.commandButton
					.removeFocusListener(this.basicPopupButtonListener);
			this.commandButton
					.removeChangeListener(this.basicPopupButtonListener);
		}

		this.commandButton
				.removePropertyChangeListener(this.propertyChangeListener);
		this.propertyChangeListener = null;

		this.commandButton
				.removeActionListener(this.disposePopupsActionListener);
		this.disposePopupsActionListener = null;

		if (this.commandButton instanceof JCommandButton) {
			((JCommandButton) this.commandButton).getPopupModel()
					.removePopupActionListener(this.popupActionListener);
			this.popupActionListener = null;
		}
	}

	/**
	 * Uninstalls the keyboard actions from the associated command button.
	 */
	protected void uninstallKeyboardActions() {
		if (this.basicPopupButtonListener != null) {
			this.basicPopupButtonListener
					.uninstallKeyboardActions(this.commandButton);
		}
	}

	/**
	 * Invoked by <code>installUI</code> to create a layout manager object to
	 * manage the {@link JCommandButtonStrip}.
	 * 
	 * @return a layout manager object
	 */
	protected LayoutManager createLayoutManager() {
		return new CommandButtonLayout();
	}

	/**
	 * Layout for the command button.
	 * 
	 * @author Kirill Grouchnikov
	 */
	private class CommandButtonLayout implements LayoutManager {
		/**
		 * Creates new layout manager.
		 */
		public CommandButtonLayout() {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#addLayoutComponent(java.lang.String,
		 * java.awt.Component)
		 */
		public void addLayoutComponent(String name, Component c) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#removeLayoutComponent(java.awt.Component)
		 */
		public void removeLayoutComponent(Component c) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#preferredLayoutSize(java.awt.Container)
		 */
		public Dimension preferredLayoutSize(Container c) {
			Insets borderInsets = commandButton.getInsets();
			int bx = borderInsets.left + borderInsets.right;
			int by = borderInsets.top + borderInsets.bottom;

			Dimension iconDim = toTakeSavedDimension ? savedDimension
					: new Dimension(commandButton.getIcon().getIconWidth(),
							commandButton.getIcon().getIconHeight());

			JSeparator jsep = new JSeparator(JSeparator.VERTICAL);
			int layoutGap = getLayoutGap();

			FontMetrics fm = c.getFontMetrics(c.getFont());
			switch (commandButton.getState()) {
			case COMPACT:
				// no label, only icon and action arrow icon
				return new Dimension(bx
						+ commandButton.getIcon().getIconWidth()
						+ ((popupActionIcon != null) ? layoutGap
								+ jsep.getPreferredSize().width
								+ fm.getHeight() / 2 : 0), by
						+ Math.max(iconDim.height, fm.getHeight()));
			case SMALL:
				// no label, only icon and action arrow icon
				return new Dimension(bx
						+ commandButton.getIcon().getIconWidth()
						+ ((popupActionIcon != null) ? 2 * layoutGap
								+ jsep.getPreferredSize().width
								+ fm.getHeight() : 0), by
						+ Math.max(iconDim.height, fm.getHeight()));
			case MEDIUM:
			case TILE:
				// icon, one-line label and action-arrow icon
				int titleWidth = fm.stringWidth(commandButton.getText());
				String extraText = commandButton.getExtraText();
				int extraWidth = fm.stringWidth((extraText == null) ? ""
						: extraText);
				double textWidth = (commandButton.getState() == ElementState.MEDIUM) ? titleWidth
						: Math.max(titleWidth, extraWidth);
				int widthMed = iconDim.width
						+ 2
						* layoutGap
						+ jsep.getPreferredSize().width
						+ (int) textWidth
						+ ((popupActionIcon != null) ? fm.getHeight() + 2
								* layoutGap + jsep.getPreferredSize().width : 0);
				// System.out.println(ribbonButton.getText() + ":" + widthMed);
				int textLines = 1 + ((commandButton.getState() == ElementState.TILE) ? 1
						: 0);
				return new Dimension(bx + widthMed, by
						+ Math.max(iconDim.height, textLines
								* (fm.getAscent() + fm.getDescent())
								+ (textLines - 1) * layoutGap));
			case CUSTOM:
			case BIG:
			case ORIG:
				// icon, two labels and action-arrow (on the second label)
				int title1Width = fm.stringWidth(titlePart1);
				int title2Width = fm.stringWidth(titlePart2);

				int width = Math.max(iconDim.width, Math.max(title1Width,
						title2Width
								+ 2
								* layoutGap
								+ jsep.getPreferredSize().width
								+ ((popupActionIcon != null) ? fm.getHeight()
										: 0)));

				int height = by + iconDim.height + layoutGap
						+ jsep.getPreferredSize().width;
				if (commandButton.getText() != null) {
					height += (2 * fm.getAscent() + fm.getDescent());
				}
				return new Dimension(bx + width, height);

			case BIG_FIXED_LANDSCAPE:
				// icon, label
				int fillTitleWidth = fm.stringWidth(commandButton.getText());

				int widthFull = Math.max(iconDim.width, fillTitleWidth);

				int heightFull = by + iconDim.height + layoutGap
						+ jsep.getPreferredSize().width;
				if (commandButton.getText() != null) {
					heightFull += fm.getHeight();
				}

				widthFull = Math.max(widthFull, heightFull * 5 / 4);
				return new Dimension(bx + widthFull, heightFull);
			}
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#minimumLayoutSize(java.awt.Container)
		 */
		public Dimension minimumLayoutSize(Container c) {
			return this.preferredLayoutSize(c);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#layoutContainer(java.awt.Container)
		 */
		public void layoutContainer(Container c) {
			ElementState state = commandButton.getState();
			int height = state.getPreferredIconSize();
			if (height > 0) {
				commandButton.getIcon().setHeight(state.getPreferredIconSize());
			} else {
				if (state == ElementState.ORIG)
					commandButton.getIcon().revertToOriginalDimension();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#update(java.awt.Graphics,
	 * javax.swing.JComponent)
	 */
	@Override
	public void update(Graphics g, JComponent c) {
		Graphics2D g2d = (Graphics2D) g.create();
		RenderingUtils.installDesktopHints(g2d);
		super.update(g2d, c);
		g2d.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#paint(java.awt.Graphics,
	 * javax.swing.JComponent)
	 */
	@Override
	public void paint(Graphics g, JComponent c) {
		if (this.isPaintingBackground()) {
			this.paintButtonBackground(g, new Rectangle(0, 0, commandButton
					.getWidth(), commandButton.getHeight()));
		}

		g.setFont(FlamingoUtilities.getFont(commandButton, "Ribbon.font",
				"Button.font", "Panel.font"));

		JCommandButton.CommandButtonKind buttonKind = (commandButton instanceof JCommandButton) ? ((JCommandButton) commandButton)
				.getCommandButtonKind()
				: JCommandButton.CommandButtonKind.ACTION_ONLY;

		this.actionClickArea = new Rectangle(0, 0, 0, 0);
		this.popupClickArea = new Rectangle(0, 0, 0, 0);

		Insets ins = commandButton.getInsets();

		Rectangle iconRect = new Rectangle();
		Rectangle bigLabel1Rect = new Rectangle();
		Rectangle bigLabel2Rect = new Rectangle();
		Rectangle midLabelRect = new Rectangle();
		Rectangle midExtraLabelRect = new Rectangle();
		Rectangle popupActionRect = new Rectangle();

		layoutButtonComponents(g, buttonKind, iconRect, bigLabel1Rect,
				bigLabel2Rect, midLabelRect, midExtraLabelRect, popupActionRect);

		if (iconRect != null) {
			this.paintButtonIcon(g, iconRect);
		}
		if (popupActionRect.getWidth() > 0) {
			paintPopupActionIcon(g, popupActionRect);
		}
		FontMetrics fm = g.getFontMetrics();

		boolean isTextPaintedEnabled = commandButton.isEnabled();
		if (commandButton instanceof JCommandButton) {
			JCommandButton jCommandButton = (JCommandButton) commandButton;
			CommandButtonKind commandButtonKind = jCommandButton
					.getCommandButtonKind();
			boolean toTakePopupModel = ((commandButtonKind == CommandButtonKind.ACTION_AND_POPUP_MAIN_POPUP) || (commandButtonKind == CommandButtonKind.POPUP_ONLY));
			ElementState commandButtonState = commandButton.getState();
			if ((commandButtonState == ElementState.CUSTOM)
					|| (commandButtonState == ElementState.BIG)
					|| (commandButtonState == ElementState.ORIG)) {
				toTakePopupModel = true;
			}

			if (toTakePopupModel) {
				// System.out.println("Taking popup model");
				isTextPaintedEnabled = jCommandButton.getPopupModel()
						.isEnabled();
			} else {
				// System.out.println("Taking action model");
				isTextPaintedEnabled = jCommandButton.getActionModel()
						.isEnabled();
			}
		}

		if (isTextPaintedEnabled) {
			g.setColor(FlamingoUtilities.getColor(Color.black,
					"Button.foreground"));
		} else {
			g.setColor(FlamingoUtilities.getColor(Color.gray,
					"Label.disabledForeground"));
		}
		if (bigLabel1Rect.getWidth() > 0) {
			int mnemonicChar = -1;// this.commandButton.getMnemonic();
			BasicGraphicsUtils.drawString(g, titlePart1, mnemonicChar,
					bigLabel1Rect.x, bigLabel1Rect.y + fm.getAscent());
		}
		if (bigLabel2Rect.getWidth() > 0) {
			int mnemonicChar = -1;// this.commandButton.getMnemonic();
			BasicGraphicsUtils.drawString(g, titlePart2, mnemonicChar,
					bigLabel2Rect.x, bigLabel2Rect.y + fm.getAscent());
		}
		if (midLabelRect.getWidth() > 0) {
			int mnemonicChar = -1;// this.commandButton.getMnemonic();
			String str = this.commandButton.getText();
			str = FlamingoUtilities.clipString(g.getFontMetrics(),
					midLabelRect.width, str);
			int xDelta = 0;
			if (commandButton.getState() == ElementState.BIG_FIXED_LANDSCAPE) {
				int width = g.getFontMetrics().stringWidth(str);
				int availableWidth = commandButton.getWidth() - ins.left
						- ins.right;
				xDelta = (availableWidth - width) / 2;
			}
			BasicGraphicsUtils.drawString(g, str, mnemonicChar, xDelta
					+ midLabelRect.x, midLabelRect.y + fm.getAscent());
		}
		if (isTextPaintedEnabled) {
			g.setColor(FlamingoUtilities.getColor(Color.gray,
					"Label.disabledForeground"));
		} else {
			g.setColor(FlamingoUtilities.getColor(Color.gray,
					"Label.disabledForeground").brighter());
		}
		if (midExtraLabelRect.getWidth() > 0) {
			BasicGraphicsUtils.drawString(g, this.commandButton.getExtraText(),
					-1, midExtraLabelRect.x, midExtraLabelRect.y
							+ fm.getAscent());
		}

		if (buttonKind.hasAction() && buttonKind.hasPopup()
				&& this.isPaintingSeparators() && (actionClickArea.width > 0)
				&& (popupClickArea.width > 0)) {
			switch (commandButton.getState()) {
			case CUSTOM:
			case BIG:
			case ORIG:
				this.paintButtonHorizontalSeparator(g, actionClickArea.height);
				break;
			default:
				this.paintButtonVerticalSeparator(g, actionClickArea.width);
			}
		}
		// Graphics2D g2d = (Graphics2D) g.create();
		// g2d.setColor(Color.red);
		// g2d.draw(bigLabel1Rect);
		// g2d.draw(bigLabel2Rect);
		// g2d.setColor(Color.blue);
		// g2d.draw(midExtraLabelRect);
		// g2d.draw(midLabelRect);
		// g2d.draw(actionArrowLabelRect);
		// g2d.dispose();
	}

	/**
	 * Paints the icon of the popup area.
	 * 
	 * @param g
	 *            Graphics context.
	 * @param actionArrowRect
	 */
	protected void paintPopupActionIcon(Graphics g, Rectangle actionArrowRect) {
		int width = actionArrowRect.width / 2;
		if (this.commandButton.getState() == ElementState.COMPACT)
			width *= 2;
		if (width % 2 == 0)
			width--;
		int height = actionArrowRect.height / 2 - 1;
		popupActionIcon.setDimension(new Dimension(width, height));
		popupActionIcon.paintIcon(this.commandButton, g, actionArrowRect.x
				+ (actionArrowRect.width - width) / 2, actionArrowRect.y
				+ (actionArrowRect.height - height) / 2);
	}

	/**
	 * Lays out the visuals of the command button.
	 * 
	 * @param g
	 *            Graphics context.
	 * @param buttonKind
	 *            Button kind.
	 * @param iconRect
	 *            Rectangle that will contain the bounds of
	 *            {@link AbstractCommandButton#getIcon()} when this method
	 *            returns.
	 * @param bigLabel1Rect
	 *            Rectangle that will contain the bounds of {@link #titlePart1}
	 *            when this method returns.
	 * @param bigLabel2Rect
	 *            Rectangle that will contain the bounds of {@link #titlePart2}
	 *            when this method returns.
	 * @param midLabelRect
	 *            Rectangle that will contain the bounds of
	 *            {@link AbstractCommandButton#getText()} when this method
	 *            returns.
	 * @param midExtraLabelRect
	 *            Rectangle that will contain the bounds of
	 *            {@link AbstractCommandButton#getExtraText()} when this method
	 *            returns.
	 * @param popupActionRect
	 *            Rectangle that will contain the bounds of popup action icon
	 *            when this method returns.
	 */
	protected void layoutButtonComponents(Graphics g,
			JCommandButton.CommandButtonKind buttonKind, Rectangle iconRect,
			Rectangle bigLabel1Rect, Rectangle bigLabel2Rect,
			Rectangle midLabelRect, Rectangle midExtraLabelRect,
			Rectangle popupActionRect) {
		int width = commandButton.getWidth();
		int height = commandButton.getHeight();

		Insets ins = commandButton.getInsets();
		int x = ins.left;
		int y = ins.top;

		FontMetrics fm = g.getFontMetrics();
		int labelHeight = fm.getAscent() + fm.getDescent();

		if (buttonKind == JCommandButton.CommandButtonKind.ACTION_ONLY) {
			actionClickArea.x = 0;
			actionClickArea.y = 0;
			actionClickArea.width = width;
			actionClickArea.height = height;
		}
		if (buttonKind == JCommandButton.CommandButtonKind.POPUP_ONLY) {
			popupClickArea.x = 0;
			popupClickArea.y = 0;
			popupClickArea.width = width;
			popupClickArea.height = height;
		}

		JSeparator jsep = new JSeparator(JSeparator.VERTICAL);
		int layoutGap = getLayoutGap();

		ResizableIcon buttonIcon = this.commandButton.getIcon();
		switch (commandButton.getState()) {
		case CUSTOM:
		case BIG:
		case ORIG:
			// Big icon, 2-line text (with possibly empty second line) and
			// action arrow on the 2nd line.

			// special case - title is null
			if (commandButton.getText() == null) {
				y = ins.top
						+ (height - ins.top - ins.bottom - buttonIcon
								.getIconHeight()) / 2;
			}
			iconRect.x = (width - buttonIcon.getIconWidth()) / 2;
			iconRect.y = y;
			iconRect.width = buttonIcon.getIconWidth();
			iconRect.height = buttonIcon.getIconHeight();
			y += buttonIcon.getIconHeight();

			if (buttonKind.hasAction() && buttonKind.hasPopup()) {
				actionClickArea.x = 0;
				actionClickArea.y = 0;
				actionClickArea.width = width;
				actionClickArea.height = y + layoutGap;

				popupClickArea.x = 0;
				popupClickArea.y = y + layoutGap;
				popupClickArea.width = width;
				popupClickArea.height = height - y - layoutGap;
			}

			y += jsep.getPreferredSize().width;

			int labelWidth = (int) fm.getStringBounds(this.titlePart1, g)
					.getWidth();

			bigLabel1Rect.x = ins.left
					+ (width - labelWidth - ins.left - ins.right) / 2;
			bigLabel1Rect.y = y;
			bigLabel1Rect.width = labelWidth;
			bigLabel1Rect.height = labelHeight;

			y += labelHeight;

			labelWidth = (int) fm.getStringBounds(this.titlePart2, g)
					.getWidth();

			int extraWidth = (popupActionIcon != null) ? 2 * layoutGap
					+ labelHeight : 0;

			x = ins.left
					+ (width - labelWidth - extraWidth - ins.left - ins.right)
					/ 2;

			bigLabel2Rect.x = x;
			bigLabel2Rect.y = y;
			bigLabel2Rect.width = labelWidth;
			bigLabel2Rect.height = labelHeight;

			if (popupActionIcon != null) {
				x += layoutGap;
				x += labelWidth;

				popupActionRect.x = x;
				popupActionRect.y = y;
				popupActionRect.width = labelHeight;
				popupActionRect.height = labelHeight;
			}

			break;

		case BIG_FIXED_LANDSCAPE:
			// Big icon, 1-line text below it

			// special case - title is null
			if (commandButton.getText() == null) {
				y = ins.top
						+ (height - ins.top - ins.bottom - buttonIcon
								.getIconHeight()) / 2;
			}
			iconRect.x = (width - buttonIcon.getIconWidth()) / 2;
			iconRect.y = y;
			iconRect.width = buttonIcon.getIconWidth();
			iconRect.height = buttonIcon.getIconHeight();
			y += buttonIcon.getIconHeight();

			y += jsep.getPreferredSize().width;

			midLabelRect.x = x;
			midLabelRect.y = y;
			midLabelRect.width = fm.stringWidth(this.commandButton.getText());
			if ((midLabelRect.x + midLabelRect.width) > (this.commandButton
					.getWidth() - ins.right)) {
				midLabelRect.width = this.commandButton.getWidth() - ins.right
						- x;
			}
			midLabelRect.height = labelHeight;

			break;

		case TILE:
			// small icon, 1-line text, 1-line extra text and action arrow
			iconRect.x = x;
			iconRect.y = (height - buttonIcon.getIconHeight()) / 2;
			iconRect.width = buttonIcon.getIconWidth();
			iconRect.height = buttonIcon.getIconHeight();

			x += commandButton.getIcon().getIconWidth();
			if (// this.isPaintingSeparators()
			// &&
			(buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_POPUP)) {
				actionClickArea.x = 0;
				actionClickArea.y = 0;
				actionClickArea.width = x + layoutGap;
				actionClickArea.height = height;

				popupClickArea.x = x + layoutGap;
				popupClickArea.y = 0;
				popupClickArea.width = width - x - layoutGap;
				popupClickArea.height = height;
			}
			x += 2 * layoutGap + jsep.getPreferredSize().width;

			midLabelRect.x = x;
			midLabelRect.y = (height - 2 * labelHeight - layoutGap) / 2;
			midLabelRect.width = fm.stringWidth(this.commandButton.getText());
			midLabelRect.height = labelHeight;

			String extraText = this.commandButton.getExtraText();
			midExtraLabelRect.x = x;
			midExtraLabelRect.y = midLabelRect.y + labelHeight + layoutGap;
			midExtraLabelRect.width = fm.stringWidth((extraText == null) ? ""
					: extraText);
			midExtraLabelRect.height = labelHeight;

			x += (int) Math.max(fm.getStringBounds(
					this.commandButton.getText(), g).getWidth(), fm
					.getStringBounds((extraText == null) ? "" : extraText, g)
					.getWidth());
			if (// this.isPaintingSeparators()
			// &&
			(buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_ACTION)) {
				actionClickArea.x = 0;
				actionClickArea.y = 0;
				actionClickArea.width = x + layoutGap;
				actionClickArea.height = height;

				popupClickArea.x = x + layoutGap;
				popupClickArea.y = 0;
				popupClickArea.width = width - x - layoutGap;
				popupClickArea.height = height;
			}
			x += 2 * layoutGap + jsep.getPreferredSize().width;

			if (popupActionIcon != null) {
				popupActionRect.x = x;
				popupActionRect.y = (height - labelHeight) / 2;
				popupActionRect.width = labelHeight;
				popupActionRect.height = labelHeight;
				x += popupActionRect.width;
			}

			x += ins.right;
			if (commandButton.getHorizontalAlignment() == SwingConstants.CENTER) {
				// at this point, we may have extra space left due to more
				// width than was required. Need to move the popup action icon
				// to the right and center the other areas
				int extraTileSpace = width - x;
				if (extraTileSpace > 0) {
					if (popupActionIcon != null) {
						popupActionRect.x += extraTileSpace;
					}
					if (buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_ACTION) {
						popupClickArea.x += extraTileSpace;
						actionClickArea.width += extraTileSpace;
					}
					if (buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_POPUP) {
						popupClickArea.x += extraTileSpace / 2;
						actionClickArea.width += extraTileSpace / 2;
					}
					iconRect.x += extraTileSpace / 2;
					midLabelRect.x += extraTileSpace / 2;
					midExtraLabelRect.x += extraTileSpace / 2;
				}
			}

			break;

		case MEDIUM:
			// small icon, 1-line text and action arrow
			iconRect.x = x;
			iconRect.y = (height - buttonIcon.getIconHeight()) / 2;
			iconRect.width = buttonIcon.getIconWidth();
			iconRect.height = buttonIcon.getIconHeight();

			x += commandButton.getIcon().getIconWidth();
			if (// this.isPaintingSeparators()
			// &&
			(buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_POPUP)) {
				actionClickArea.x = 0;
				actionClickArea.y = 0;
				actionClickArea.width = x + layoutGap;
				actionClickArea.height = height;

				popupClickArea.x = x + layoutGap;
				popupClickArea.y = 0;
				popupClickArea.width = width - x - layoutGap;
				popupClickArea.height = height;
			}
			x += 2 * layoutGap + jsep.getPreferredSize().width;

			midLabelRect.x = x;
			midLabelRect.y = (height - labelHeight) / 2;
			midLabelRect.width = fm.stringWidth(this.commandButton.getText());
			midLabelRect.height = labelHeight;

			x += (int) fm.getStringBounds(this.commandButton.getText(), g)
					.getWidth();
			if (// this.isPaintingSeparators()
			// &&
			(buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_ACTION)) {
				actionClickArea.x = 0;
				actionClickArea.y = 0;
				actionClickArea.width = x + layoutGap;
				actionClickArea.height = height;

				popupClickArea.x = x + layoutGap;
				popupClickArea.y = 0;
				popupClickArea.width = width - x - layoutGap;
				popupClickArea.height = height;
			}
			x += 2 * layoutGap + jsep.getPreferredSize().width;

			if (popupActionIcon != null) {
				popupActionRect.x = x;
				popupActionRect.y = (height - labelHeight) / 2;
				popupActionRect.width = labelHeight;
				popupActionRect.height = labelHeight;
				x += popupActionRect.width;
			}

			x += ins.right;
			if (commandButton.getHorizontalAlignment() == SwingConstants.CENTER) {
				// at this point, we may have extra space left due to more
				// width than was required. Need to move the popup action icon
				// to the right and center the other areas
				int extraMediumSpace = width - x;
				if (extraMediumSpace > 0) {
					if (popupActionIcon != null) {
						popupActionRect.x += extraMediumSpace;
					}
					if (buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_ACTION) {
						popupClickArea.x += extraMediumSpace;
						actionClickArea.width += extraMediumSpace;
					}
					if (buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_POPUP) {
						popupClickArea.x += extraMediumSpace / 2;
						actionClickArea.width += extraMediumSpace / 2;
					}
					iconRect.x += extraMediumSpace / 2;
					midLabelRect.x += extraMediumSpace / 2;
					midExtraLabelRect.x += extraMediumSpace / 2;
				}
			}

			break;

		case SMALL:
			// small icon and action arrow
			iconRect.x = x;
			iconRect.y = (height - buttonIcon.getIconHeight()) / 2;
			iconRect.width = buttonIcon.getIconWidth();
			iconRect.height = buttonIcon.getIconHeight();

			x += commandButton.getIcon().getIconWidth();
			if (buttonKind.hasAction() && buttonKind.hasPopup()) {
				// && this.isPaintingSeparators()) {
				actionClickArea.x = 0;
				actionClickArea.y = 0;
				actionClickArea.width = x + layoutGap;
				actionClickArea.height = height;

				popupClickArea.x = x + layoutGap;
				popupClickArea.y = 0;
				popupClickArea.width = width - x - layoutGap;
				popupClickArea.height = height;
			}
			x += 2 * layoutGap + jsep.getPreferredSize().width;

			if (popupActionIcon != null) {
				popupActionRect.x = x;
				popupActionRect.y = (height - labelHeight) / 2;
				popupActionRect.width = labelHeight;
				popupActionRect.height = labelHeight;
				x += popupActionRect.width;
			}

			x += ins.right;
			if (commandButton.getHorizontalAlignment() == SwingConstants.CENTER) {
				// at this point, we may have extra space left due to more
				// width than was required. Need to move the popup action icon
				// to the right and center the other areas
				int extraSmallSpace = width - x;
				if (extraSmallSpace > 0) {
					if (popupActionIcon != null) {
						popupActionRect.x += extraSmallSpace;
					}
					if (buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_ACTION) {
						popupClickArea.x += extraSmallSpace;
						actionClickArea.width += extraSmallSpace;
					}
					if (buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_POPUP) {
						popupClickArea.x += extraSmallSpace / 2;
						actionClickArea.width += extraSmallSpace / 2;
					}
					iconRect.x += extraSmallSpace / 2;
					midLabelRect.x += extraSmallSpace / 2;
					midExtraLabelRect.x += extraSmallSpace / 2;
				}
			}

			break;
		case COMPACT:
			// small icon and action arrow
			iconRect.x = x;
			iconRect.y = (height - buttonIcon.getIconHeight()) / 2;
			iconRect.width = buttonIcon.getIconWidth();
			iconRect.height = buttonIcon.getIconHeight();

			x += commandButton.getIcon().getIconWidth();
			if (buttonKind.hasAction() && buttonKind.hasPopup()) {
				// && this.isPaintingSeparators()) {
				actionClickArea.x = 0;
				actionClickArea.y = 0;
				actionClickArea.width = x + layoutGap / 2;
				actionClickArea.height = height;

				popupClickArea.x = x + layoutGap / 2;
				popupClickArea.y = 0;
				popupClickArea.width = width - x - layoutGap / 2;
				popupClickArea.height = height;
			}
			x += layoutGap + jsep.getPreferredSize().width - 1;

			if (popupActionIcon != null) {
				popupActionRect.x = x;
				popupActionRect.y = (height - labelHeight) / 2;
				popupActionRect.width = labelHeight / 2;
				popupActionRect.height = labelHeight;
				x += popupActionRect.width;
			}

			x += ins.right;
			if (commandButton.getHorizontalAlignment() == SwingConstants.CENTER) {
				// at this point, we may have extra space left due to more
				// width than was required. Need to move the popup action icon
				// to the right and center the other areas
				int extraSmallSpace = width - x;
				if (extraSmallSpace > 0) {
					if (popupActionIcon != null) {
						popupActionRect.x += extraSmallSpace;
					}
					if (buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_ACTION) {
						popupClickArea.x += extraSmallSpace;
						actionClickArea.width += extraSmallSpace;
					}
					if (buttonKind == JCommandButton.CommandButtonKind.ACTION_AND_POPUP_MAIN_POPUP) {
						popupClickArea.x += extraSmallSpace / 2;
						actionClickArea.width += extraSmallSpace / 2;
					}
					iconRect.x += extraSmallSpace / 2;
					midLabelRect.x += extraSmallSpace / 2;
					midExtraLabelRect.x += extraSmallSpace / 2;
				}
			}

			break;
		}
		commandButton.putClientProperty("icon.bounds", iconRect);
	}

	/**
	 * Returns the current icon.
	 * 
	 * @return Current icon.
	 */
	protected Icon getIconToPaint() {
		// special case for command buttons with POPUP_ONLY kind -
		// check the popup model
		boolean toUseDisabledIcon;
		if (this.commandButton instanceof JCommandButton
				&& ((JCommandButton) this.commandButton).getCommandButtonKind() == JCommandButton.CommandButtonKind.POPUP_ONLY) {
			toUseDisabledIcon = !((JCommandButton) this.commandButton)
					.getPopupModel().isEnabled();
		} else {
			toUseDisabledIcon = !this.commandButton.getActionModel()
					.isEnabled();
		}
		return (toUseDisabledIcon && this.commandButton.getDisabledIcon() != null) ? this.commandButton
				.getDisabledIcon()
				: this.commandButton.getIcon();
	}

	/**
	 * Paints command button vertical separator.
	 * 
	 * @param graphics
	 *            Graphics context.
	 * @param separatorX
	 *            X coordinate of the separator.
	 */
	protected void paintButtonVerticalSeparator(Graphics graphics,
			int separatorX) {
		this.buttonRendererPane.setBounds(0, 0, this.commandButton.getWidth(),
				this.commandButton.getHeight());
		Graphics2D g2d = (Graphics2D) graphics.create();
		this.rendererSeparator.setOrientation(JSeparator.VERTICAL);
		this.buttonRendererPane.paintComponent(g2d, this.rendererSeparator,
				this.commandButton, separatorX, 2, 2, this.commandButton
						.getHeight() - 4, true);
		g2d.dispose();
	}

	/**
	 * Paints command button horizontal separator.
	 * 
	 * @param graphics
	 *            Graphics context.
	 * @param separatorY
	 *            Y coordinate of the separator.
	 */
	protected void paintButtonHorizontalSeparator(Graphics graphics,
			int separatorY) {
		this.buttonRendererPane.setBounds(0, 0, this.commandButton.getWidth(),
				this.commandButton.getHeight());
		Graphics2D g2d = (Graphics2D) graphics.create();
		this.rendererSeparator.setOrientation(JSeparator.HORIZONTAL);
		this.buttonRendererPane.paintComponent(g2d, this.rendererSeparator,
				this.commandButton, 2, separatorY, this.commandButton
						.getWidth() - 4, 2, true);
		g2d.dispose();
	}

	/**
	 * Paints command button background.
	 * 
	 * @param graphics
	 *            Graphics context.
	 * @param toFill
	 *            Rectangle for the background.
	 */
	protected void paintButtonBackground(Graphics graphics, Rectangle toFill) {
		ButtonModel actionModel = this.commandButton.getActionModel();
		PopupButtonModel popupModel = (this.commandButton instanceof JCommandButton) ? ((JCommandButton) this.commandButton)
				.getPopupModel()
				: null;

		// first time - paint the full background passing both models
		this.paintButtonBackground(graphics, toFill, actionModel, popupModel);
		Rectangle actionArea = this.getActionClickArea();
		Rectangle popupArea = this.getPopupClickArea();
		if ((actionArea != null) && !actionArea.isEmpty()) {
			// now overlay the action area with the background matching action
			// model
			Graphics2D graphicsAction = (Graphics2D) graphics.create();
			// System.out.println(actionArea);
			graphicsAction.clip(actionArea);
			float actionAlpha = 0.4f;
			if ((popupModel != null) && !popupModel.isEnabled())
				actionAlpha = 1.0f;
			graphicsAction.setComposite(AlphaComposite.SrcOver
					.derive(actionAlpha));
			// System.out.println(graphicsAction.getClipBounds());
			this.paintButtonBackground(graphicsAction, toFill, actionModel);
			graphicsAction.dispose();
		}
		if ((popupArea != null) && !popupArea.isEmpty()) {
			// now overlay the popup area with the background matching popup
			// model
			Graphics2D graphicsPopup = (Graphics2D) graphics.create();
			// System.out.println(popupArea);
			graphicsPopup.clip(popupArea);
			// System.out.println(graphicsPopup.getClipBounds());
			float popupAlpha = 0.4f;
			if (!actionModel.isEnabled())
				popupAlpha = 1.0f;
			graphicsPopup.setComposite(AlphaComposite.SrcOver
					.derive(popupAlpha));
			this.paintButtonBackground(graphicsPopup, toFill, popupModel);
			graphicsPopup.dispose();
		}
	}

	/**
	 * Paints the background of the command button.
	 * 
	 * @param graphics
	 *            Graphics context.
	 * @param toFill
	 *            Rectangle to fill.
	 * @param modelToUse
	 *            Button models to use for computing the background fill.
	 */
	protected void paintButtonBackground(Graphics graphics, Rectangle toFill,
			ButtonModel... modelToUse) {
		if (modelToUse.length == 0)
			return;
		if ((modelToUse.length == 1) && (modelToUse[0] == null))
			return;

		this.buttonRendererPane.setBounds(toFill.x, toFill.y, toFill.width,
				toFill.height);
		this.rendererButton.setRolloverEnabled(true);
		boolean isEnabled = true;
		boolean isRollover = false;
		boolean isPressed = true;
		boolean isArmed = true;
		boolean isSelected = true;
		for (ButtonModel model : modelToUse) {
			if (model == null)
				continue;
			isEnabled = isEnabled && model.isEnabled();
			isRollover = isRollover || model.isRollover();
			isPressed = isPressed && model.isPressed();
			isArmed = isArmed && model.isArmed();
			isSelected = isSelected && model.isSelected();
		}
		this.rendererButton.getModel().setEnabled(isEnabled);
		this.rendererButton.getModel().setRollover(isRollover);
		this.rendererButton.getModel().setPressed(isPressed);
		this.rendererButton.getModel().setArmed(isArmed);
		this.rendererButton.getModel().setSelected(isSelected);
		// System.out.println(this.commandButton.getText() + " - e:"
		// + this.rendererButton.getModel().isEnabled() + ", s:"
		// + this.rendererButton.getModel().isSelected() + ", r:"
		// + this.rendererButton.getModel().isRollover() + ", p:"
		// + this.rendererButton.getModel().isPressed() + ", a:"
		// + this.rendererButton.getModel().isArmed());
		Graphics2D g2d = (Graphics2D) graphics.create();

		if (Boolean.TRUE.equals(this.commandButton
				.getClientProperty(EMULATE_SQUARE_BUTTON))) {
			this.buttonRendererPane.paintComponent(g2d, this.rendererButton,
					this.commandButton, toFill.x - toFill.width / 2, toFill.y
							- toFill.height / 2, 2 * toFill.width,
					2 * toFill.height, true);
			g2d.setColor(FlamingoUtilities.getColor(Color.gray,
					"TextField.inactiveForeground", "Button.disabledText",
					"ComboBox.disabledForeground"));
			g2d.drawRect(toFill.x, toFill.y, toFill.width - 1,
					toFill.height - 1);
		} else {
			this.buttonRendererPane.paintComponent(g2d, this.rendererButton,
					this.commandButton, toFill.x, toFill.y, toFill.width,
					toFill.height, true);
		}
		g2d.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.jvnet.flamingo.common.ui.CommandButtonUI#updateState(org.jvnet.flamingo
	 * .common.ElementState, boolean)
	 */
	@Override
	public void updateState(ElementState state, boolean toUpdateIcon) {
		int maxHeight = 0;
		switch (state) {
		case COMPACT:
		case SMALL:
		case MEDIUM:
		case BIG:
		case BIG_FIXED_LANDSCAPE:
		case TILE:
			maxHeight = state.getPreferredIconSize();
			break;
		case ORIG:
			maxHeight = this.commandButton.getIcon().getIconHeight();
			break;
		}

		toTakeSavedDimension = !toUpdateIcon && (state != ElementState.CUSTOM);
		savedDimension = new Dimension(maxHeight, maxHeight);
		if (toUpdateIcon) {
			if (state != ElementState.CUSTOM) {
				this.commandButton.getIcon().setHeight(maxHeight);
				// also update disabled icon
				if (this.commandButton.getDisabledIcon() != null) {
					this.commandButton.getDisabledIcon().setHeight(maxHeight);
				}
			}
		}
	}

	/**
	 * Updates the custom dimension.
	 */
	protected void updateCustomDimension() {
		int dimension = this.commandButton.getCustomDimension();

		if (dimension > 0) {
			this.commandButton.getIcon().setDimension(
					new Dimension(dimension, dimension));
			this.commandButton.setState(ElementState.CUSTOM, true);

			this.commandButton.doLayout();
			this.commandButton.repaint();
		}
	}

	/**
	 * Updates the popup action icon.
	 */
	protected void updatePopupActionIcon() {
		JCommandButton button = (JCommandButton) this.commandButton;
		if ((button.getPopupPanel() != null)
				|| button.getCommandButtonKind().hasPopup()) {
			this.popupActionIcon = this.createPopupActionIcon();
		} else {
			this.popupActionIcon = null;
		}
	}

	/**
	 * Paints the button icon.
	 * 
	 * @param g
	 *            Graphics context.
	 * @param iconRect
	 *            Icon rectangle.
	 */
	protected void paintButtonIcon(Graphics g, Rectangle iconRect) {
		Icon iconToPaint = this.getIconToPaint();
		if ((iconRect != null) && (iconToPaint != null)) {
			iconToPaint
					.paintIcon(this.commandButton, g, iconRect.x, iconRect.y);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.swing.plaf.ComponentUI#getPreferredSize(javax.swing.JComponent)
	 */
	@Override
	public Dimension getPreferredSize(JComponent c) {
		return c.getLayout().preferredLayoutSize(c);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jvnet.flamingo.common.ui.CommandButtonUI#getActionClickArea()
	 */
	@Override
	public Rectangle getActionClickArea() {
		return this.actionClickArea;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jvnet.flamingo.common.ui.CommandButtonUI#getPopupClickArea()
	 */
	@Override
	public Rectangle getPopupClickArea() {
		return this.popupClickArea;
	}

	/**
	 * Returns the layout gap for the visuals of the associated command button.
	 * 
	 * @return The layout gap for the visuals of the associated command button.
	 */
	protected int getLayoutGap() {
		Font font = this.commandButton.getFont();
		if (font == null)
			font = UIManager.getFont("Button.font");
		return (font.getSize() - 4) / 4;
	}

	/**
	 * Returns indication whether the action-popup areas separator is painted.
	 * 
	 * @return <code>true</code> if the action-popup areas separator is painted.
	 */
	protected boolean isPaintingSeparators() {
		PopupButtonModel popupModel = (this.commandButton instanceof JCommandButton) ? ((JCommandButton) this.commandButton)
				.getPopupModel()
				: null;
		boolean isActionRollover = this.commandButton.getActionModel()
				.isRollover();
		boolean isPopupRollover = (popupModel != null)
				&& popupModel.isRollover();
		return isActionRollover || isPopupRollover;
	}

	/**
	 * Returns indication whether the button background is painted.
	 * 
	 * @return <code>true</code> if the button background is painted.
	 */
	protected boolean isPaintingBackground() {
		PopupButtonModel popupModel = (this.commandButton instanceof JCommandButton) ? ((JCommandButton) this.commandButton)
				.getPopupModel()
				: null;
		boolean isActionSelected = this.commandButton.getActionModel()
				.isSelected();
		boolean isPopupSelected = (popupModel != null)
				&& popupModel.isSelected();
		boolean isActionRollover = this.commandButton.getActionModel()
				.isRollover();
		boolean isPopupRollover = (popupModel != null)
				&& popupModel.isRollover();

		return (isActionSelected || isPopupSelected || isActionRollover
				|| isPopupRollover || !this.commandButton.isFlat());
	}

	/**
	 * Creates the popup action listener for this command button.
	 * 
	 * @return Popup action listener for this command button.
	 */
	protected PopupActionListener createPopupActionListener() {
		return new PopupActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// dismiss all the popups that are currently showing
				// up until <this> button.
				PopupPanelManager.defaultManager().hidePopups(commandButton);

				if (!(commandButton instanceof JCommandButton))
					return;

				// check if the command button has an associated popup
				// panel
				final JPopupPanel popupPanel = ((JCommandButton) commandButton)
						.getPopupPanel();
				if (popupPanel != null) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							PopupFactory popupFactory = PopupFactory
									.getSharedInstance();

							if ((commandButton == null) || (popupPanel == null)
									|| popupPanel.isEmpty())
								return;
							int x = commandButton.getLocationOnScreen().x;
							int y = commandButton.getLocationOnScreen().y
									+ commandButton.getSize().height;
							// System.out.println("Showing "
							// + jgb.gallery.getComponent());

							// make sure that the popup gallery stays in bounds
							Rectangle scrBounds = commandButton
									.getGraphicsConfiguration().getBounds();
							int pw = popupPanel.getPreferredSize().width;
							if ((x + pw) > (scrBounds.x + scrBounds.width)) {
								x = scrBounds.x + scrBounds.width - pw;
							}
							int ph = popupPanel.getPreferredSize().height;
							if ((y + ph) > (scrBounds.y + scrBounds.height)) {
								y = scrBounds.y + scrBounds.height - ph;
							}

							// get the popup and show it
							Popup popup = popupFactory.getPopup(commandButton,
									popupPanel, x, y);
							PopupPanelManager.defaultManager().addPopup(
									commandButton, popup, popupPanel);
							// popupButtons.put(popup, jgb);
							// popupGalleries.put(popup, jgb.gallery);
						}
					});
					return;
				}

				// otherwise - create a popup menu and ask all the listeners
				// to populate it
				JPopupMenu popupMenu = new JPopupMenu();
				PopupMenuListener[] listeners = commandButton
						.getListeners(PopupMenuListener.class);
				if (listeners != null) {
					// Process the listeners notifying
					// those that are interested in this event
					for (PopupMenuListener listener : listeners) {
						listener.menuAboutToShow(popupMenu);
					}
				}
				popupMenu.show(commandButton, 0, commandButton.getHeight());
			}
		};
	}
}
