/*
 * Copyright (c) 2005-2008 Flamingo Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Flamingo Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.flamingo.ribbon;

import java.awt.*;

import javax.swing.JFrame;
import javax.swing.JRootPane;

import org.jvnet.flamingo.ribbon.ui.JRibbonRootPane;

/**
 * Ribbon frame. Provides the same functionality as a regular {@link JFrame},
 * but with a {@link JRibbon} component in the {@link BorderLayout#NORTH}
 * location. In version 4.0 this will be the only way to have a ribbon -
 * {@link JRibbon} constructor will be made package-protected. The
 * implementation enforces that a {@link JRibbon} component is always at the
 * {@link BorderLayout#NORTH} location, throwing
 * {@link IllegalArgumentException} on attempts to set a custom layout manager,
 * add another component at {@link BorderLayout#NORTH} or remove the
 * {@link JRibbon} component.
 * 
 * 
 * @author Kirill Grouchnikov
 */
public class JRibbonFrame extends JFrame {
	/**
	 * The ribbon component.
	 */
	private JRibbon ribbon;

	/**
	 * Custom layout manager that enforces the {@link JRibbon} location at
	 * {@link BorderLayout#NORTH}.
	 * 
	 * @author Kirill Grouchnikov
	 */
	private class RibbonFrameLayout extends BorderLayout {
		@Override
		public void addLayoutComponent(Component comp, Object constraints) {
			if (constraints.equals(BorderLayout.NORTH)) {
				if (getLayoutComponent(BorderLayout.NORTH) != null) {
					throw new IllegalArgumentException(
							"Already has a NORTH JRibbon component");
				}
				if (!(comp instanceof JRibbon)) {
					throw new IllegalArgumentException(
							"Can't add non-JRibbon component to NORTH location");
				}
			}
			super.addLayoutComponent(comp, constraints);
		}

		@Override
		public void removeLayoutComponent(Component comp) {
			if (comp instanceof JRibbon) {
				throw new IllegalArgumentException(
						"Can't remove JRibbon component");
			}
			super.removeLayoutComponent(comp);
		}
	}

	/**
	 * Creates a new ribbon frame with no title.
	 * 
	 * @throws HeadlessException
	 *             If GraphicsEnvironment.isHeadless() returns true.
	 */
	public JRibbonFrame() throws HeadlessException {
		super();
		this.initRibbon();
	}

	/**
	 * Creates a new ribbon frame with no title.
	 * 
	 * @param gc
	 *            Graphics configuration to use.
	 */
	public JRibbonFrame(GraphicsConfiguration gc) {
		super(gc);
		this.initRibbon();
	}

	/**
	 * Creates a new ribbon frame with the specified title.
	 * 
	 * @param title
	 *            Ribbon frame title.
	 * @throws HeadlessException
	 *             If GraphicsEnvironment.isHeadless() returns true.
	 */
	public JRibbonFrame(String title) throws HeadlessException {
		super(title);
		this.initRibbon();
	}

	/**
	 * Creates a new ribbon frame with the specified title.
	 * 
	 * @param title
	 *            Ribbon frame title.
	 * @param gc
	 *            Graphics configuration to use.
	 * @throws HeadlessException
	 *             If GraphicsEnvironment.isHeadless() returns true.
	 */
	public JRibbonFrame(String title, GraphicsConfiguration gc) {
		super(title, gc);
		this.initRibbon();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JFrame#setLayout(java.awt.LayoutManager)
	 */
	@Override
	public void setLayout(LayoutManager manager) {
		if (manager.getClass() != RibbonFrameLayout.class) {
			LayoutManager currManager = getLayout();
			if (currManager != null) {
				throw new IllegalArgumentException(
						"Can't set a custom layout manager on JRibbonFrame");
			}
		}
		super.setLayout(manager);
	}

	/**
	 * Initializes the layout and the ribbon.
	 */
	private void initRibbon() {
		this.setLayout(new RibbonFrameLayout());
		this.ribbon = new JRibbon();
		this.add(this.ribbon, BorderLayout.NORTH);
	}

	/**
	 * Returns the ribbon component.
	 * 
	 * @return Ribbon component.
	 */
	public JRibbon getRibbon() {
		return this.ribbon;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JFrame#createRootPane()
	 */
	@Override
	protected JRootPane createRootPane() {
		JRootPane rp = new JRibbonRootPane();
		rp.setOpaque(true);
		return rp;
	}
}
