/*
 * Copyright (c) 2005-2008 Flamingo Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Flamingo Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.flamingo.ribbon.ui;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.*;

import org.jvnet.flamingo.ribbon.*;
import org.jvnet.flamingo.utils.FlamingoUtilities;
import org.jvnet.flamingo.utils.RenderingUtils;

/**
 * Basic UI for ribbon {@link JRibbon}.
 * 
 * @author Kirill Grouchnikov
 */
public class BasicRibbonUI extends RibbonUI {
	/**
	 * Minimum width of a single ribbon band.
	 */
	private static final int BAND_MIN_WIDTH = 30;

	/**
	 * Client property marking the ribbon component to indicate whether the task
	 * bar and contextual task group headers should be shown on the title pane
	 * of the window. This is only relevant for the {@link JRibbonFrame}.
	 */
	public static final String IS_USING_TITLE_PANE = "ribbon.internal.isUsingTitlePane";

	/**
	 * The associated ribbon.
	 */
	protected JRibbon ribbon;

	/**
	 * Mouse wheel listener to switch between ribbon tasks.
	 */
	protected MouseWheelListener mouseWheelListener;

	/**
	 * Taskbar panel.
	 */
	protected JPanel taskBarPanel;

	/**
	 * Map of toggle buttons of all tasks.
	 */
	protected Map<RibbonTask, JRibbonTaskToggleButton> taskToggleButtons;

	/**
	 * Button group for task toggle buttons.
	 */
	protected ButtonGroup taskToggleButtonGroup;

	/**
	 * Change listener.
	 */
	protected ChangeListener ribbonChangeListener;

	/**
	 * Property change listener.
	 */
	protected PropertyChangeListener propertyChangeListener;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#createUI(javax.swing.JComponent)
	 */
	public static ComponentUI createUI(JComponent c) {
		return new BasicRibbonUI();
	}

	/**
	 * Creates a new basic ribbon UI delegate.
	 */
	public BasicRibbonUI() {
		this.taskToggleButtons = new HashMap<RibbonTask, JRibbonTaskToggleButton>();
		this.taskToggleButtonGroup = new ButtonGroup();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#installUI(javax.swing.JComponent)
	 */
	@Override
	public void installUI(JComponent c) {
		this.ribbon = (JRibbon) c;
		installDefaults();
		installComponents();
		installListeners();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#uninstallUI(javax.swing.JComponent)
	 */
	@Override
	public void uninstallUI(JComponent c) {
		uninstallListeners();
		uninstallComponents();
		uninstallDefaults();

		this.ribbon = null;
	}

	/**
	 * Installs listeners on the associated ribbon.
	 */
	protected void installListeners() {
		this.mouseWheelListener = new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				// invoke only when mouse is in the tab area
				PointerInfo pInfo = MouseInfo.getPointerInfo();
				Point pLocation = pInfo.getLocation();
				SwingUtilities.convertPointFromScreen(pLocation, ribbon);
				if (pLocation.getY() > getTaskbarHeight())
					return;

				int delta = e.getWheelRotation();
				if (delta == 0)
					return;

				// get the list of all currently showing tasks
				final List<RibbonTask> visibleTasks = getCurrentlyShownRibbonTasks();
				// find the index of the currently selected task
				int currSelectedTaskIndex = visibleTasks.indexOf(ribbon
						.getSelectedTask());
				// compute the next task

				int newSelectedTaskIndex = currSelectedTaskIndex
						+ ((delta > 0) ? 1 : -1);
				if (newSelectedTaskIndex < 0)
					return;
				if (newSelectedTaskIndex >= visibleTasks.size())
					return;

				final int indexToSet = newSelectedTaskIndex;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						ribbon.setCursor(Cursor
								.getPredefinedCursor(Cursor.WAIT_CURSOR));
						ribbon.setSelectedTask(visibleTasks.get(indexToSet));
						ribbon.setCursor(Cursor
								.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					}
				});
			}
		};
		this.ribbon.addMouseWheelListener(this.mouseWheelListener);

		this.ribbonChangeListener = new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				syncTaskToggleButtons();
			}
		};
		this.ribbon.addChangeListener(this.ribbonChangeListener);

		this.propertyChangeListener = new PropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				if ("selectedTask".equals(evt.getPropertyName())) {
					RibbonTask old = (RibbonTask) evt.getOldValue();
					RibbonTask curr = (RibbonTask) evt.getNewValue();
					if ((old != null) && (taskToggleButtons.get(old) != null)) {
						taskToggleButtons.get(old).setSelected(false);
					}
					if ((curr != null) && (taskToggleButtons.get(curr) != null)) {
						taskToggleButtons.get(curr).setSelected(true);
					}
				}
			}
		};
		this.ribbon.addPropertyChangeListener(this.propertyChangeListener);
	}

	/**
	 * Uninstalls listeners from the associated ribbon.
	 */
	protected void uninstallListeners() {
		this.ribbon.removeMouseWheelListener(this.mouseWheelListener);
		this.mouseWheelListener = null;

		this.ribbon.removeChangeListener(this.ribbonChangeListener);
		this.ribbonChangeListener = null;

		this.ribbon.removePropertyChangeListener(this.propertyChangeListener);
		this.propertyChangeListener = null;
	}

	/**
	 * Installs defaults on the associated ribbon.
	 */
	protected void installDefaults() {
		Border b = this.ribbon.getBorder();
		if (b == null || b instanceof UIResource) {
			Border toSet = UIManager.getBorder("Ribbon.border");
			if (toSet == null)
				toSet = new BorderUIResource.EmptyBorderUIResource(1, 2, 1, 2);
			this.ribbon.setBorder(toSet);
		}
	}

	/**
	 * Uninstalls defaults from the associated ribbon.
	 */
	protected void uninstallDefaults() {
	}

	/**
	 * Installs subcomponents on the associated ribbon.
	 */
	protected void installComponents() {
		this.taskBarPanel = new JPanel();
		this.taskBarPanel.setName("JRibbon Task Bar");
		this.ribbon.add(this.taskBarPanel);
		this.ribbon.setLayout(createLayoutManager());
		this.taskBarPanel.setLayout(createTaskbarLayoutManager());

		this.syncTaskToggleButtons();
	}

	/**
	 * Uninstalls subcomponents from the associated ribbon.
	 */
	protected void uninstallComponents() {
		this.taskBarPanel.removeAll();
		this.ribbon.remove(this.taskBarPanel);
		this.taskBarPanel.setLayout(null);

		for (JRibbonTaskToggleButton button : this.taskToggleButtons.values()) {
			this.ribbon.remove(button);
			this.taskToggleButtonGroup.remove(button);
		}

		this.ribbon.setLayout(null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#update(java.awt.Graphics,
	 * javax.swing.JComponent)
	 */
	@Override
	public void update(Graphics g, JComponent c) {
		Graphics2D g2d = (Graphics2D) g.create();
		RenderingUtils.installDesktopHints(g2d);
		super.update(g2d, c);
		g2d.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#paint(java.awt.Graphics,
	 * javax.swing.JComponent)
	 */
	@Override
	public void paint(Graphics g, JComponent c) {
		this.paintBackground(g);

		Insets ins = c.getInsets();
		this.paintTaskArea(g, 0, ins.top + getTaskbarHeight(), c.getWidth(), c
				.getHeight()
				- getTaskbarHeight() - ins.top - ins.bottom);
	}

	/**
	 * Paints the ribbon background.
	 * 
	 * @param g
	 *            Graphics context.
	 */
	protected void paintBackground(Graphics g) {
		Graphics2D g2d = (Graphics2D) g.create();

		g2d.setColor(FlamingoUtilities.getColor(Color.lightGray,
				"Panel.background"));
		g2d.fillRect(0, 0, this.ribbon.getWidth(), this.ribbon.getHeight());

		g2d.dispose();
	}

	/**
	 * Paints the task border.
	 * 
	 * @param g
	 *            Graphics context.
	 * @param x
	 *            Left X of the tasks band bounds.
	 * @param y
	 *            Top Y of the tasks band bounds.
	 * @param width
	 *            Width of the tasks band bounds.
	 * @param height
	 *            Height of the tasks band bounds.
	 */
	protected void paintTaskArea(Graphics g, int x, int y, int width, int height) {
		JRibbonTaskToggleButton selectedTaskButton = this.taskToggleButtons
				.get(this.ribbon.getSelectedTask());
		Rectangle selectedTaskButtonBounds = selectedTaskButton.getBounds();
		Shape outerContour = FlamingoUtilities
				.getRibbonBorderOutline(x + 1, x + width - 3,
						selectedTaskButtonBounds.x + 1,
						selectedTaskButtonBounds.x
								+ selectedTaskButtonBounds.width - 1,
						selectedTaskButtonBounds.y, y, y + height, 2);

		Graphics2D g2d = (Graphics2D) g.create();
		g2d.setColor(FlamingoUtilities.getColor(Color.gray,
				"TextField.inactiveForeground", "Button.disabledText",
				"ComboBox.disabledForeground"));
		g2d.draw(outerContour);

		// check whether the currently selected task is a contextual task
		RibbonTask selected = this.ribbon.getSelectedTask();
		RibbonContextualTaskGroup contextualGroup = selected
				.getContextualGroup();
		if (contextualGroup != null) {
			// paint a small gradient directly below the task area
			Insets ins = this.ribbon.getInsets();
			int topY = ins.top + getTaskbarHeight();
			int bottomY = topY + 5;
			Color hueColor = contextualGroup.getHueColor();
			Paint paint = new GradientPaint(0, topY, FlamingoUtilities
					.getAlphaColor(hueColor,
							(int) (255 * RibbonContextualTaskGroup.HUE_ALPHA)),
					0, bottomY, FlamingoUtilities.getAlphaColor(hueColor, 0));
			g2d.setPaint(paint);
			g2d.clip(outerContour);
			g2d.fillRect(0, topY, width, bottomY - topY + 1);
		}

		// paint outlines of the contextual task groups
		paintContextualTaskGroupsOutlines(g);

		g2d.dispose();
	}

	/**
	 * Paints the outline of the contextual task groups.
	 * 
	 * @param g
	 *            Graphics context.
	 */
	protected void paintContextualTaskGroupsOutlines(Graphics g) {
		for (int i = 0; i < this.ribbon.getContextualTaskGroupCount(); i++) {
			RibbonContextualTaskGroup group = this.ribbon
					.getContextualTaskGroup(i);
			if (!ribbon.isVisible(group))
				continue;
			// go over all the tasks in this group and compute the union
			// of bounds of the matching tab buttons
			Rectangle rect = getContextualTaskGroupBounds(group);
			this.paintContextualTaskGroupOutlines(g, group, rect);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.jvnet.flamingo.ribbon.ui.RibbonUI#getContextualGroupTabBounds(org
	 * .jvnet.flamingo.ribbon.RibbonContextualTaskGroup)
	 */
	@Override
	public Rectangle getContextualTaskGroupBounds(
			RibbonContextualTaskGroup group) {
		Rectangle rect = null;
		for (int j = 0; j < group.getTaskCount(); j++) {
			JRibbonTaskToggleButton button = taskToggleButtons.get(group.getTask(j));
			if (rect == null)
				rect = button.getBounds();
			else
				rect = rect.union(button.getBounds());
		}
		int buttonGap = getTabButtonGap();
		return new Rectangle(rect.x - buttonGap / 3, rect.y, rect.width
				+ buttonGap * 2 / 3, rect.height);
	}

	/**
	 * Paints the outline of the specified contextual task group.
	 * 
	 * @param g
	 *            Graphics context.
	 * @param group
	 *            Contextual task group.
	 * @param groupBounds
	 *            Contextual task group bounds.
	 */
	protected void paintContextualTaskGroupOutlines(Graphics g,
			RibbonContextualTaskGroup group, Rectangle groupBounds) {
		Graphics2D g2d = (Graphics2D) g.create();
		Color color = FlamingoUtilities.getColor(Color.gray,
				"TextField.inactiveForeground", "Button.disabledText",
				"ComboBox.disabledForeground");

		Paint paint = new GradientPaint(0, groupBounds.y, color, 0,
				groupBounds.y + groupBounds.height, FlamingoUtilities
						.getAlphaColor(color, 0));
		g2d.setPaint(paint);
		// left line
		int x = groupBounds.x;
		g2d.drawLine(x, groupBounds.y, x, groupBounds.y + groupBounds.height);
		// right line
		x = groupBounds.x + groupBounds.width;
		g2d.drawLine(x, groupBounds.y, x, groupBounds.y + groupBounds.height);

		g2d.dispose();
	}

	/**
	 * Returns the layout gap for the bands in the associated ribbon.
	 * 
	 * @return The layout gap for the bands in the associated ribbon.
	 */
	protected int getBandGap() {
		return 2;
	}

	/**
	 * Returns the layout gap for the tab buttons in the associated ribbon.
	 * 
	 * @return The layout gap for the tab buttons in the associated ribbon.
	 */
	protected int getTabButtonGap() {
		return 6;
	}

	/**
	 * Invoked by <code>installUI</code> to create a layout manager object to
	 * manage the {@link JRibbon}.
	 * 
	 * @return a layout manager object
	 */
	protected LayoutManager createLayoutManager() {
		return new RibbonLayout();
	}

	/**
	 * Invoked by <code>installUI</code> to create a layout manager object to
	 * manage the {@link JRibbon} taskbar.
	 * 
	 * @return a layout manager object
	 */
	protected LayoutManager createTaskbarLayoutManager() {
		return new TaskbarLayout();
	}

	/**
	 * Returns the height of the taskbar area.
	 * 
	 * @return The height of the taskbar area.
	 */
	protected int getTaskbarHeight() {
		return 22;
	}

	/**
	 * Layout for the ribbon.
	 * 
	 * @author Kirill Grouchnikov
	 */
	private class RibbonLayout implements LayoutManager {
		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#addLayoutComponent(java.lang.String,
		 * java.awt.Component)
		 */
		public void addLayoutComponent(String name, Component c) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#removeLayoutComponent(java.awt.Component)
		 */
		public void removeLayoutComponent(Component c) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#preferredLayoutSize(java.awt.Container)
		 */
		public Dimension preferredLayoutSize(Container c) {
			Insets ins = c.getInsets();
			int maxPrefBandHeight = 0;
			RibbonTask selectedTask = ribbon.getSelectedTask();
			for (JRibbonBand ribbonBand : selectedTask.getBands()) {
				int bandPrefHeight = ribbonBand.getPreferredSize().height;
				Insets bandInsets = ribbonBand.getInsets();
				maxPrefBandHeight = Math.max(maxPrefBandHeight, bandPrefHeight
						+ bandInsets.top + bandInsets.bottom);
			}

			return new Dimension(c.getWidth(), maxPrefBandHeight
					+ getTaskbarHeight() + ins.top + ins.bottom);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#minimumLayoutSize(java.awt.Container)
		 */
		public Dimension minimumLayoutSize(Container c) {
			// go over all ribbon bands and sum the width
			// of ribbon buttons (of collapsed state)
			Insets ins = c.getInsets();
			int width = 0;
			int maxMinBandHeight = 0;
			int gap = getBandGap();

			RibbonTask selectedTask = ribbon.getSelectedTask();
			for (JRibbonBand ribbonBand : selectedTask.getBands()) {
				int bandPrefHeight = ribbonBand.getMinimumSize().height;
				Insets bandInsets = ribbonBand.getInsets();
				int availableBandHeight = c.getHeight() - ins.top
						- getTaskbarHeight()
						// - bandInsets.top
						// - bandInsets.bottom
						- ins.bottom;
				BasicRibbonBandUI bandUI = (BasicRibbonBandUI) ribbonBand
						.getUI();
				width += bandUI.getPreferredWidth(RibbonBandCollapseKind.ICON,
						availableBandHeight);
				maxMinBandHeight = Math.max(maxMinBandHeight, bandPrefHeight
						+ bandInsets.top + bandInsets.bottom);
			}
			// add inter-band gaps
			width += gap * (selectedTask.getBandCount() - 1);
			return new Dimension(width, maxMinBandHeight + getTaskbarHeight()
					+ ins.top + ins.bottom);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#layoutContainer(java.awt.Container)
		 */
		public void layoutContainer(Container c) {
			Insets ins = c.getInsets();
			int gap = getBandGap();
			int tabButtonGap = getTabButtonGap();

			// the top row - regular components and task buttons
			int x = ins.left;
			int taskbarHeight = getTaskbarHeight();

			// handle taskbar only if it is not marked
			if (!Boolean.TRUE.equals(ribbon
					.getClientProperty(IS_USING_TITLE_PANE))) {
				taskBarPanel.removeAll();
				for (Component regComp : ribbon.getTaskbarComponents()) {
					taskBarPanel.add(regComp);
				}
				taskBarPanel.setBounds(x, ins.top, taskBarPanel
						.getPreferredSize().width, taskbarHeight);
				x += taskBarPanel.getPreferredSize().width;
			}

			// task buttons
			int totalTaskButtonsWidth = 0;
			int helpTaskButtonWidth = 0;
			List<RibbonTask> visibleTasks = getCurrentlyShownRibbonTasks();
			RibbonTask helpTask = ribbon.getHelpTask();
			for (RibbonTask task : visibleTasks) {
				JRibbonTaskToggleButton tabButton = taskToggleButtons.get(task);
				if (task == helpTask) {
					helpTaskButtonWidth = tabButton.getPreferredSize().width;
				} else {
					int pw = tabButton.getPreferredSize().width;
					totalTaskButtonsWidth += (pw + tabButtonGap);
				}
			}

			switch (ribbon.getAlignment()) {
			case SwingConstants.LEFT:
				break;
			case SwingConstants.RIGHT:
				x = Math.max(x, c.getWidth() - totalTaskButtonsWidth
						- helpTaskButtonWidth);
				break;
			case SwingConstants.CENTER:
				x += Math.max(x, (c.getWidth() - totalTaskButtonsWidth) / 2
						- helpTaskButtonWidth);
			}

			// compute bounds for the tab buttons
			for (RibbonTask task : visibleTasks) {
				JRibbonTaskToggleButton tabButton = taskToggleButtons.get(task);
				int pw = tabButton.getPreferredSize().width;
				if (task == helpTask) {
					tabButton.setBounds(c.getWidth() - helpTaskButtonWidth
							- tabButtonGap, ins.top, pw, taskbarHeight);
				} else {
					tabButton.setBounds(x, ins.top, pw, taskbarHeight);
					x += (pw + tabButtonGap);
				}
			}

			RibbonTask selectedTask = ribbon.getSelectedTask();

			// Ask each band how much horizontal space it needs
			// under various constraints. After that, go over total
			// width under each constraint and find the optimal
			// allocation of space.
			Map<RibbonBandCollapseKind, Integer> widths = new HashMap<RibbonBandCollapseKind, Integer>();
			for (RibbonBandCollapseKind collapseKind : RibbonBandCollapseKind
					.values()) {
				int totalWidth = 0;
				for (JRibbonBand ribbonBand : selectedTask.getBands()) {
					// Insets bandInsets = band.getInsets();
					int availableBandHeight = c.getHeight() - ins.top
							- getTaskbarHeight() -
							// bandInsets.top
							// - bandInsets.bottom -
							ins.bottom;
					BasicRibbonBandUI ui = (BasicRibbonBandUI) ribbonBand
							.getUI();
					totalWidth += ui.getPreferredWidth(collapseKind,
							availableBandHeight);
				}
				totalWidth += gap * (selectedTask.getBandCount() - 1);
				widths.put(collapseKind, totalWidth);
			}

			RibbonBandCollapseKind bestFitCollapse = RibbonBandCollapseKind
					.getSortedKinds().getLast();
			int availableWidth = c.getWidth() - ins.left - ins.right;
			for (RibbonBandCollapseKind collapseKind : RibbonBandCollapseKind
					.getSortedKinds()) {
				if (widths.get(collapseKind) < availableWidth) {
					bestFitCollapse = collapseKind;
					break;
				}
			}
			// System.out.println(bestFitCollapse + " is best");
			int bestFitWidth = widths.get(bestFitCollapse);
			// double coef = (double) (availableWidth) / bestFitWidth;
			// System.out.println("available : " + c.getWidth());
			// maximum 20% more than preferred width
			// if (coef >= 1.2)
			// coef = 1.2;

			// compute how many pixels we can distribute between the bands
			int widthToDistribute = availableWidth - bestFitWidth;

			x = 1 + ins.left;
			for (JRibbonBand ribbonBand : selectedTask.getBands()) {
				BasicRibbonBandUI ui = (BasicRibbonBandUI) ribbonBand.getUI();

				// Insets bandInsets = band.getInsets();
				int availableBandHeight = c.getHeight() - ins.top
						- getTaskbarHeight() // - bandInsets.top
						// - bandInsets.bottom
						- ins.bottom;

				int pw = Math.max(BAND_MIN_WIDTH, ui.getPreferredWidth(
						bestFitCollapse, availableBandHeight));

				// try allocating the available pixels to the current band and
				// see if the collapse kind for this specific band will change.
				int availableForBand = pw + widthToDistribute;
				RibbonBandCollapseKind bestFitForBand = RibbonBandCollapseKind
						.getSortedKinds().getLast();
				for (RibbonBandCollapseKind collapseKind : RibbonBandCollapseKind
						.getSortedKinds()) {
					if (ui.getPreferredWidth(collapseKind, availableBandHeight) <= availableForBand) {
						bestFitForBand = collapseKind;
						break;
					}
				}

				// now that the collapse kind of the band is computed according
				// to the actual available width, compute the preferred band
				// width under that collapse kind
				int fw = ui.getPreferredWidth(bestFitForBand,
						availableBandHeight);

				// and update the available width accordingly.
				widthToDistribute -= (fw - pw);

				Insets bandInsets = ribbonBand.getInsets();
				ribbonBand.setBounds(x, ins.top + taskbarHeight
						+ bandInsets.top, fw, c.getHeight() - ins.top
						- taskbarHeight - bandInsets.top - bandInsets.bottom
						- ins.bottom);
				ribbonBand.doLayout();

				RibbonBandCollapseKind actualCollapseKind = ui
						.getCurrentFitCollapse();
				if (actualCollapseKind != bestFitForBand) {
					throw new IllegalStateException("Available width of '"
							+ ribbonBand.getTitle()
							+ "' under "
							+ bestFitForBand.name()
							+ " was reported to be "
							+ fw
							+ ", but the actual collapse kind is "
							+ actualCollapseKind
							+ " with width "
							+ ui.getPreferredWidth(actualCollapseKind,
									availableBandHeight));
				}

				x += (fw + gap);
			}
		}
	}

	/**
	 * Layout for the task bar.
	 * 
	 * @author Kirill Grouchnikov
	 */
	private class TaskbarLayout implements LayoutManager {
		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#addLayoutComponent(java.lang.String,
		 * java.awt.Component)
		 */
		public void addLayoutComponent(String name, Component c) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#removeLayoutComponent(java.awt.Component)
		 */
		public void removeLayoutComponent(Component c) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#preferredLayoutSize(java.awt.Container)
		 */
		public Dimension preferredLayoutSize(Container c) {
			Insets ins = c.getInsets();
			int pw = 0;
			int gap = getBandGap();
			for (Component regComp : ribbon.getTaskbarComponents()) {
				pw += regComp.getPreferredSize().width;
				pw += gap;
			}
			return new Dimension(pw + ins.left + ins.right, getTaskbarHeight()
					+ ins.top + ins.bottom);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#minimumLayoutSize(java.awt.Container)
		 */
		public Dimension minimumLayoutSize(Container c) {
			return this.preferredLayoutSize(c);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.LayoutManager#layoutContainer(java.awt.Container)
		 */
		public void layoutContainer(Container c) {
			Insets ins = c.getInsets();
			int gap = getBandGap();

			int x = ins.left;
			int taskbarHeight = getTaskbarHeight();

			for (Component regComp : ribbon.getTaskbarComponents()) {
				int pw = regComp.getPreferredSize().width;
				regComp.setBounds(x, ins.top, pw, taskbarHeight);
				x += (pw + gap);
			}
		}
	}

	/**
	 * Synchronizes the collection of tab toggle buttons with the currently
	 * visible ribbon tasks (general and contextual).
	 */
	protected void syncTaskToggleButtons() {
		// remove all the existing tab toggle buttons
		for (JRibbonTaskToggleButton button : this.taskToggleButtons.values()) {
			this.ribbon.remove(button);
			this.taskToggleButtonGroup.remove(button);
		}

		// go over all visible ribbon tasks and create a toggle button
		// for each one of them
		List<RibbonTask> visibleTasks = this.getCurrentlyShownRibbonTasks();
		for (final RibbonTask task : visibleTasks) {
			JRibbonTaskToggleButton taskToggleButton = new JRibbonTaskToggleButton(
					task.getTitle());
			taskToggleButton.setSelected(this.ribbon.getSelectedTask() == task);
			// wire listener to select the task when the button is
			// selected
			taskToggleButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							ribbon.setSelectedTask(task);
						}
					});
				}
			});
			// set the background hue color on the tab buttons
			// of tasks in contextual groups
			if (task.getContextualGroup() != null) {
				taskToggleButton.setContextualGroupHueColor(task
						.getContextualGroup().getHueColor());
			}

			this.taskToggleButtonGroup.add(taskToggleButton);
			this.ribbon.add(taskToggleButton);
			this.taskToggleButtons.put(task, taskToggleButton);
		}
	}

	/**
	 * Returns the list of currently shown ribbon tasks. This method is for
	 * internal use only.
	 * 
	 * @return The list of currently shown ribbon tasks.
	 */
	protected List<RibbonTask> getCurrentlyShownRibbonTasks() {
		List<RibbonTask> result = new ArrayList<RibbonTask>();
		RibbonTask helpTask = ribbon.getHelpTask();

		// add all regular tasks except the help task
		for (int i = 0; i < this.ribbon.getTaskCount(); i++) {
			RibbonTask task = this.ribbon.getTask(i);
			if (task != helpTask)
				result.add(task);
		}
		// add all tasks of visible contextual groups
		for (int i = 0; i < this.ribbon.getContextualTaskGroupCount(); i++) {
			RibbonContextualTaskGroup group = this.ribbon
					.getContextualTaskGroup(i);
			if (this.ribbon.isVisible(group)) {
				for (int j = 0; j < group.getTaskCount(); j++) {
					RibbonTask task = group.getTask(j);
					result.add(task);
				}
			}
		}
		// add help task
		if (helpTask != null)
			result.add(helpTask);

		return result;
	}
}
